package com.xhkjedu.sapi.service.classreport;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.sapi.mapper.classreport.*;
import com.xhkjedu.sapi.model.classreport.*;
import com.xhkjedu.utils.DatesUtil;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.sapi.vo.classreport.CGradeSubjectVo;
import com.xhkjedu.sapi.vo.classreport.ReportClasstypeVo;
import com.xhkjedu.sapi.vo.classreport.ReportRateVo;
import com.xhkjedu.sapi.vo.classreport.YearOnYearVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author ywx
 * @classname ClassReportService
 * @description 班级报告
 * @date 2021/5/26 9:55
 **/
@Service
@Slf4j
public class ClassReportService {
    @Resource
    private ClassReportMapper classReportMapper;
    @Resource
    private ReportClasstypeMapper reportClasstypeMapper;
    @Resource
    private ReportClassGuideMapper reportClassGuideMapper;
    @Resource
    private ReportClassPaperMapper reportClassPaperMapper;
    @Resource
    private ReportClassReciteMapper reportClassReciteMapper;
    @Resource
    private ReportClassRoomMapper reportClassRoomMapper;
    @Resource
    private ReportClassErrorqueMapper reportClassErrorqueMapper;
    @Resource
    private ReportClassErrorquefdMapper reportClassErrorquefdMapper;
    @Resource
    private ReportClassLiveMapper reportClassLiveMapper;

    public Map<String,List<ReportClasstypeVo>> parseList(List<ReportClasstypeVo> list, Integer type, boolean now){
        if (!now && type == 1) {//班级
            return list.stream().collect(Collectors.groupingBy(m->m.getClasstype()+"_"+m.getClassid()+"_"+m.getSubjectid()
                    ,LinkedHashMap::new, Collectors.toList()));
        } else if (!now && type == 2) {//年级
            return list.stream().collect(Collectors.groupingBy(m->m.getSchoolid()+"_"+m.getClasstype()+"_"+m.getGradeid()+"_"+m.getSubjectid()
                    ,LinkedHashMap::new, Collectors.toList()));
        } else if (now && type == 1) {//班级日期
            return list.stream().collect(Collectors.groupingBy(m->m.getClasstype()+"_"+m.getClassid()+"_"+m.getSubjectid()
                    +"_"+m.getBegindate(),LinkedHashMap::new, Collectors.toList()));
        } else {//年级日期
            return list.stream().collect(Collectors.groupingBy(m->m.getSchoolid()+"_"+m.getClasstype()+"_"+m.getGradeid()+"_"+m.getSubjectid()
                    +"_"+m.getBegindate(),LinkedHashMap::new, Collectors.toList()));
        }
    }

    //保存班级报告类型
    public Integer saveType(TReportClasstype type, ReportClasstypeVo vo){
        TReportClasstype type2 = new TReportClasstype();
        type2.setClassid(vo.getClassid());
        type2.setGradeid(vo.getGradeid());
        type2.setSubjectid(vo.getSubjectid());
        type2.setYear(vo.getYear());
        type2.setSchoolid(vo.getSchoolid());
        type2.setCreatetime(N_Utils.getSecondTimestamp());
        type2.setReporttype(type.getReporttype());
        type2.setBegindate(type.getBegindate());
        type2.setEnddate(type.getEnddate());
        reportClasstypeMapper.insertUseGeneratedKeys(type2);
        return type2.getId();
    }

    public void setTypeDate(TReportClasstype type) {
        String begindate;
        String enddate;
        Integer basenum = type.getBasenum()==null?0:type.getBasenum();
        type.setBasenum(basenum);
        if (type.getReporttype() == 1) {
            begindate = DatesUtil.getBeginDayOfLastWeek(basenum);
            enddate = DatesUtil.getEndDayOfLastWeek(basenum);
        } else {
            begindate = DatesUtil.getBeginDayOfLastMonth(basenum);
            enddate = DatesUtil.getEndDayOfLastMonth(basenum);
        }
        type.setBegindate(begindate);
        type.setEnddate(enddate);
    }

    //生成班级报告
    public void generateClassReport(TReportClasstype type){
        setTypeDate(type);

        //预习
        List<ReportClasstypeVo> yxlist = classReportMapper.listPreview(type);
        savePreview(type, yxlist, false);

        //课堂
        List<ReportClasstypeVo> ktlist = classReportMapper.listClassroom(type);
        saveClassroom(type, ktlist, false);
        
        //作业
        List<ReportClasstypeVo> zylist = classReportMapper.listPaper(type);
        savePaper(type, zylist, false);

        //背诵
        List<ReportClasstypeVo> bslist = classReportMapper.listRecite(type);
        saveRecite(type, bslist, false);

        //错题辅导
        List<ReportClasstypeVo> fdlist = classReportMapper.listErrQuestionFd(type);
        saveErrQuestionFd(type, fdlist, false);

        //错题
        List<ReportClasstypeVo> ctlist = classReportMapper.listErrQuestion(type);
        saveErrQuestion(type, ctlist, false);

        //直播
        List<ReportClasstypeVo> zblist = classReportMapper.listLive(type);
        saveLive(type, zblist, false);
    }

    public List<TReportClassGuide> savePreview(TReportClasstype type, List<ReportClasstypeVo> list, boolean now) {
        List<TReportClassGuide> list1 = new ArrayList<>();
        Map<String, ReportRateVo> maxnum = new HashMap<>();//年级科目下最高发布次数
        Map<String,Integer> fbrank = new HashMap<>();//年级科目下发布次数排名
        Map<String,Integer> yxrank = new HashMap<>();//年级科目下预习率排名
        Map<String, List<ReportClasstypeVo>> gsubjects = parseList(list, 2, now);
        for(Map.Entry s:gsubjects.entrySet()) {
            List<CGradeSubjectVo> gslist = new ArrayList<>();
            List<ReportClasstypeVo> gsubjects2 = (List<ReportClasstypeVo>) s.getValue();
            Map<Integer,List<ReportClasstypeVo>> clist = gsubjects2.stream().collect(Collectors.groupingBy(m->m.getClassid()));
            for(Map.Entry c:clist.entrySet()) {
                List<ReportClasstypeVo> clist2 = (List<ReportClasstypeVo>) c.getValue();
                ReportClasstypeVo vo = clist2.get(0);
                CGradeSubjectVo gs = new CGradeSubjectVo();
                gs.setClassid(vo.getClassid());
                gs.setClasstype(vo.getClasstype());
                gs.setGradeid(vo.getGradeid());
                gs.setSubjectid(vo.getSubjectid());
                gs.setMaxnum(clist2.stream().mapToInt(ReportClasstypeVo::getFbnum).sum());
                int xsum = clist2.stream().mapToInt(ReportClasstypeVo::getXsnum).sum();
                int tjum = clist2.stream().mapToInt(ReportClasstypeVo::getTjnum).sum();
                gs.setNumrate(N_Utils.getIntegerDivideAndMulitiply(tjum,xsum));
                gslist.add(gs);
            }
            ReportRateVo rateVo = new ReportRateVo();
            String key = (String) s.getKey();
            rateVo.setMaxnum(gslist.stream().max(Comparator.comparing(CGradeSubjectVo::getMaxnum)).get().getMaxnum());
            rateVo.setTjrate(gslist.stream().max(Comparator.comparing(CGradeSubjectVo::getNumrate)).get().getNumrate());
            maxnum.put(key,rateVo);

            //发布次数排名
            gslist = gslist.stream().sorted((s1, s2) -> -Long.compare(s1.getMaxnum(), s2.getMaxnum())).collect(Collectors.toList());
            List<Map.Entry<Integer, List<CGradeSubjectVo>>> gslist2 = gslist.stream()
                    .collect(Collectors.groupingBy(CGradeSubjectVo::getMaxnum)).entrySet()
                    .stream().sorted((s1, s2) -> -Long.compare(s1.getKey(), s2.getKey())).collect(Collectors.toList());
            Integer index = 1;
            for (Map.Entry<Integer, List<CGradeSubjectVo>> entry : gslist2) {
                for(CGradeSubjectVo cgs : entry.getValue()){
                    fbrank.put(key+"_"+cgs.getClassid(),index);
                }
                index = index + entry.getValue().size();
            }

            //预习率排名
            gslist = gslist.stream().sorted((s1, s2) -> -Double.compare(s1.getNumrate(), s2.getNumrate())).collect(Collectors.toList());
            List<Map.Entry<Double, List<CGradeSubjectVo>>> yxlist2 = gslist.stream()
                    .collect(Collectors.groupingBy(CGradeSubjectVo::getNumrate)).entrySet()
                    .stream().sorted((s1, s2) -> -Double.compare(s1.getKey(), s2.getKey())).collect(Collectors.toList());
            index = 1;
            for (Map.Entry<Double, List<CGradeSubjectVo>> entry : yxlist2) {
                for(CGradeSubjectVo cgs : entry.getValue()){
                    yxrank.put(key+"_"+cgs.getClassid(),index);
                }
                index = index + entry.getValue().size();
            }
        }
        Map<String, List<ReportClasstypeVo>> subjects = parseList(list, 1, now);
        Integer classid = type.getClassid();
        for(Map.Entry s:subjects.entrySet()){
            List<ReportClasstypeVo> slist2 = (List<ReportClasstypeVo>) s.getValue();
            DoubleSummaryStatistics statistics = slist2.stream().mapToDouble(ReportClasstypeVo::getNumrate).summaryStatistics();
            ReportClasstypeVo vo = slist2.get(0);
            if (N_Utils.isEmptyInteger(classid) || classid.equals(vo.getClassid())) {
                try {
                    TReportClassGuide rg = new TReportClassGuide();
                    rg.setClassnum(vo.getClassnum());
                    int guidenum = slist2.stream().mapToInt(ReportClasstypeVo::getFbnum).sum();
                    rg.setGuidenum(guidenum);
                    String cgsid = vo.getSchoolid() + "_" + vo.getClasstype() + "_" + vo.getGradeid() + "_" + vo.getSubjectid();
                    if (now) cgsid = cgsid + "_" + vo.getBegindate();
                    ReportRateVo rateVo = maxnum.get(cgsid);
                    rg.setGuidemaxnum(rateVo.getMaxnum());
                    String cgscid = cgsid + "_" + vo.getClassid();
                    rg.setGuiderank(fbrank.get(cgscid));
                    int xsnum = slist2.stream().mapToInt(ReportClasstypeVo::getXsnum).sum();
                    int tjnum = slist2.stream().mapToInt(ReportClasstypeVo::getTjnum).sum();
                    rg.setPrenum(tjnum);
                    Double guiderate = N_Utils.getIntegerDivideAndMulitiply(tjnum, xsnum);
                    rg.setGuiderate(guiderate);
                    rg.setGuideavgrate(N_Utils.formatDouble(statistics.getAverage(), 2));
                    rg.setGuidemaxrate(rateVo.getTjrate());
                    rg.setGuideraterank(yxrank.get(cgscid));
                    rg.setCreatetime(N_Utils.getSecondTimestamp());
                    //计算同比
                    TReportClasstype type2 = new TReportClasstype();
                    type2.setReporttype(type.getReporttype());
                    type2.setClassid(vo.getClassid());
                    type2.setSubjectid(vo.getSubjectid());
                    YearOnYearVo y = classReportMapper.getPreviewNum(type2);
                    if (y == null) {
                        rg.setGuideprerate(100.0);
                        rg.setPrenumrate(100.0);
                        rg.setPrerate(100.0);
                    } else {
                        rg.setGuideprerate(N_Utils.getIntegerDivideAndMulitiply(guidenum - y.getFbnum(), guidenum));
                        rg.setPrenumrate(N_Utils.getIntegerDivideAndMulitiply(tjnum - y.getTjnum(), tjnum));
                        rg.setPrerate(N_Utils.formatDouble(guiderate - y.getTjrate(), 2));
                    }
                    if (!now) {
                        Integer creportid = saveType(type, vo);
                        rg.setCreportid(creportid);
                        reportClassGuideMapper.insertSelective(rg);
                    } else {
                        rg.setBegindate(vo.getBegindate());
                        rg.setEnddate(vo.getBegindate());
                    }
                    list1.add(rg);
                } catch (Exception e) {
                    log.error("保存预习报告数据出错", e.getMessage());
                }
            }
        }
        return list1;
    }
    
    public List<TReportClassRoom> saveClassroom(TReportClasstype type, List<ReportClasstypeVo> list, boolean now) {
        List<TReportClassRoom> list1 = new ArrayList<>();
        Map<String,Integer> maxnum = new HashMap<>();//年级科目下最高发布次数
        Map<String,Integer> fbrank = new HashMap<>();//年级科目下发布次数排名
        Map<String,Integer> cqrank = new HashMap<>();//年级科目下出勤率排名
        Map<String, List<ReportClasstypeVo>> gsubjects = parseList(list, 2, now);
        for(Map.Entry s:gsubjects.entrySet()) {
            List<CGradeSubjectVo> gslist = new ArrayList<>();
            List<ReportClasstypeVo> gsubjects2 = (List<ReportClasstypeVo>) s.getValue();
            Map<Integer,List<ReportClasstypeVo>> clist = gsubjects2.stream().collect(Collectors.groupingBy(m->m.getClassid()));
            for(Map.Entry c:clist.entrySet()) {
                List<ReportClasstypeVo> clist2 = (List<ReportClasstypeVo>) c.getValue();
                ReportClasstypeVo vo = clist2.get(0);
                CGradeSubjectVo gs = new CGradeSubjectVo();
                gs.setClassid(vo.getClassid());
                gs.setClasstype(vo.getClasstype());
                gs.setGradeid(vo.getGradeid());
                gs.setSubjectid(vo.getSubjectid());
                gs.setMaxnum(clist2.stream().mapToInt(ReportClasstypeVo::getFbnum).sum());
                int xsum = clist2.stream().mapToInt(ReportClasstypeVo::getXsnum).sum();
                int tjum = clist2.stream().mapToInt(ReportClasstypeVo::getTjnum).sum();
                gs.setNumrate(N_Utils.getIntegerDivideAndMulitiply(tjum,xsum));
                gslist.add(gs);
            }
            String key = (String) s.getKey();
            CGradeSubjectVo vo2 = gslist.stream().max(Comparator.comparing(CGradeSubjectVo::getMaxnum)).get();
            maxnum.put(key,vo2.getMaxnum());

            //发布次数排名
            gslist = gslist.stream().sorted((s1, s2) -> -Long.compare(s1.getMaxnum(), s2.getMaxnum())).collect(Collectors.toList());
            List<Map.Entry<Integer, List<CGradeSubjectVo>>> gslist2 = gslist.stream()
                    .collect(Collectors.groupingBy(CGradeSubjectVo::getMaxnum)).entrySet()
                    .stream().sorted((s1, s2) -> -Long.compare(s1.getKey(), s2.getKey())).collect(Collectors.toList());
            Integer index = 1;
            for (Map.Entry<Integer, List<CGradeSubjectVo>> entry : gslist2) {
                for(CGradeSubjectVo cgs : entry.getValue()){
                    fbrank.put(key+"_"+cgs.getClassid(),index);
                }
                index = index + entry.getValue().size();
            }

            //预习率排名
            gslist = gslist.stream().sorted((s1, s2) -> -Double.compare(s1.getNumrate(), s2.getNumrate())).collect(Collectors.toList());
            List<Map.Entry<Double, List<CGradeSubjectVo>>> yxlist2 = gslist.stream()
                    .collect(Collectors.groupingBy(CGradeSubjectVo::getNumrate)).entrySet()
                    .stream().sorted((s1, s2) -> -Double.compare(s1.getKey(), s2.getKey())).collect(Collectors.toList());
            index = 1;
            for (Map.Entry<Double, List<CGradeSubjectVo>> entry : yxlist2) {
                for(CGradeSubjectVo cgs : entry.getValue()){
                    cqrank.put(key+"_"+cgs.getClassid(),index);
                }
                index = index + entry.getValue().size();
            }
        }
        Map<String, List<ReportClasstypeVo>> subjects = parseList(list, 1, now);
        Integer classid = type.getClassid();
        for(Map.Entry s:subjects.entrySet()){
            List<ReportClasstypeVo> slist2 = (List<ReportClasstypeVo>) s.getValue();
            DoubleSummaryStatistics statistics = slist2.stream().mapToDouble(ReportClasstypeVo::getNumrate).summaryStatistics();
            ReportClasstypeVo vo = slist2.get(0);
            if (N_Utils.isEmptyInteger(classid) || classid.equals(vo.getClassid())) {
                try {
                    TReportClassRoom rr = new TReportClassRoom();
                    rr.setClassnum(vo.getClassnum());
                    int fbnum = slist2.stream().mapToInt(ReportClasstypeVo::getFbnum).sum();
                    rr.setRoomnum(fbnum);
                    String cgsid = vo.getSchoolid() + "_" + vo.getClasstype() + "_" + vo.getGradeid() + "_" + vo.getSubjectid();
                    if (now) cgsid = cgsid + "_" + vo.getBegindate();
                    rr.setRoommaxnum(maxnum.get(cgsid));
                    String cgscid = cgsid + "_" + vo.getClassid();
                    rr.setRoomrank(fbrank.get(cgscid));
                    rr.setTwnum(vo.getTwnum());
                    rr.setTcnum(vo.getTcnum());
                    rr.setDmnum(vo.getDmnum());
                    rr.setQdnum(vo.getQdnum());
                    int xsnum = slist2.stream().mapToInt(ReportClasstypeVo::getXsnum).sum();
                    int tjnum = slist2.stream().mapToInt(ReportClasstypeVo::getTjnum).sum();
                    Double guiderate = N_Utils.getIntegerDivideAndMulitiply(tjnum, xsnum);
                    rr.setCqrate(guiderate);
                    rr.setCqavgrate(N_Utils.formatDouble(statistics.getAverage(), 2));
                    rr.setCqmaxrate(statistics.getMax());
                    rr.setCqraterank(cqrank.get(cgscid));
                    rr.setCreatetime(N_Utils.getSecondTimestamp());
                    //计算同比
                    TReportClasstype type2 = new TReportClasstype();
                    type2.setReporttype(type.getReporttype());
                    type2.setClassid(vo.getClassid());
                    type2.setSubjectid(vo.getSubjectid());
                    YearOnYearVo y = classReportMapper.getClassroomNum(type2);
                    if (y == null) {
                        rr.setCqprerate(100.0);
                        rr.setPrerate(100.0);
                    } else {
                        rr.setPrerate(N_Utils.getIntegerDivideAndMulitiply(fbnum - y.getFbnum(), fbnum));
                        rr.setCqprerate(N_Utils.formatDouble(guiderate - y.getTjrate(), 2));
                    }
                    if (!now) {
                        Integer creportid = saveType(type, vo);
                        rr.setCreportid(creportid);
                        reportClassRoomMapper.insertSelective(rr);
                    } else {
                        rr.setBegindate(vo.getBegindate());
                        rr.setEnddate(vo.getBegindate());
                    }
                    list1.add(rr);
                } catch (Exception e) {
                    log.error("保存课堂数据出错", e.getMessage());
                }
            }
        }
        return list1;
    }

    public List<TReportClassPaper> savePaper(TReportClasstype type, List<ReportClasstypeVo> list, boolean now) {
        List<TReportClassPaper> list1 = new ArrayList<>();
        Map<String, ReportRateVo> maxnum = new HashMap<>();//年级科目下最高发布次数
        Map<String,Integer> fbrank = new HashMap<>();//年级科目下发布次数排名
        Map<String,Integer> tjrank = new HashMap<>();//年级科目下提交率排名
        Map<String,Integer> pgrank = new HashMap<>();//年级科目下提交率排名
        Map<String,Integer> zqrank = new HashMap<>();//年级科目下正确率排名
        Map<String, List<ReportClasstypeVo>> gsubjects = parseList(list, 2, now);
        for(Map.Entry s:gsubjects.entrySet()) {
            List<CGradeSubjectVo> gslist = new ArrayList<>();
            List<ReportClasstypeVo> gsubjects2 = (List<ReportClasstypeVo>) s.getValue();
            Map<Integer,List<ReportClasstypeVo>> clist = gsubjects2.stream().collect(Collectors.groupingBy(m->m.getClassid()));
            for(Map.Entry c:clist.entrySet()) {
                List<ReportClasstypeVo> clist2 = (List<ReportClasstypeVo>) c.getValue();
                ReportClasstypeVo vo = clist2.get(0);
                CGradeSubjectVo gs = new CGradeSubjectVo();
                gs.setClassid(vo.getClassid());
                gs.setClasstype(vo.getClasstype());
                gs.setGradeid(vo.getGradeid());
                gs.setSubjectid(vo.getSubjectid());
                gs.setMaxnum(clist2.stream().mapToInt(ReportClasstypeVo::getFbnum).sum());
                int xsum = clist2.stream().mapToInt(ReportClasstypeVo::getXsnum).sum();
                int tjum = clist2.stream().mapToInt(ReportClasstypeVo::getTjnum).sum();
                int pgum = clist2.stream().mapToInt(ReportClasstypeVo::getPgnum).sum();
                Double pscore = clist2.stream().mapToDouble(ReportClasstypeVo::getPscore).sum();
                Double uscore = clist2.stream().mapToDouble(ReportClasstypeVo::getUscore).sum();
                gs.setNumrate(N_Utils.getIntegerDivideAndMulitiply(tjum,xsum));
                gs.setPgrate(N_Utils.getIntegerDivideAndMulitiply(pgum,tjum));
                gs.setZqrate(N_Utils.getDoubleDivideAndMulitiply(uscore,pscore));
                gslist.add(gs);
            }
            ReportRateVo rateVo = new ReportRateVo();
            String key = (String) s.getKey();
            rateVo.setMaxnum(gslist.stream().max(Comparator.comparing(CGradeSubjectVo::getMaxnum)).get().getMaxnum());
            rateVo.setTjrate(gslist.stream().max(Comparator.comparing(CGradeSubjectVo::getNumrate)).get().getNumrate());
            rateVo.setPgrate(gslist.stream().max(Comparator.comparing(CGradeSubjectVo::getPgrate)).get().getPgrate());
            rateVo.setZqrate(gslist.stream().max(Comparator.comparing(CGradeSubjectVo::getZqrate)).get().getZqrate());
            maxnum.put(key,rateVo);

            //发布次数排名
            gslist = gslist.stream().sorted((s1, s2) -> -Long.compare(s1.getMaxnum(), s2.getMaxnum())).collect(Collectors.toList());
            List<Map.Entry<Integer, List<CGradeSubjectVo>>> gslist2 = gslist.stream()
                    .collect(Collectors.groupingBy(CGradeSubjectVo::getMaxnum)).entrySet()
                    .stream().sorted((s1, s2) -> -Long.compare(s1.getKey(), s2.getKey())).collect(Collectors.toList());
            Integer index = 1;
            for (Map.Entry<Integer, List<CGradeSubjectVo>> entry : gslist2) {
                for(CGradeSubjectVo cgs : entry.getValue()){
                    fbrank.put(key+"_"+cgs.getClassid(),index);
                }
                index = index + entry.getValue().size();
            }

            //提交率排名
            gslist = gslist.stream().sorted((s1, s2) -> -Double.compare(s1.getNumrate(), s2.getNumrate())).collect(Collectors.toList());
            List<Map.Entry<Double, List<CGradeSubjectVo>>> yxlist2 = gslist.stream()
                    .collect(Collectors.groupingBy(CGradeSubjectVo::getNumrate)).entrySet()
                    .stream().sorted((s1, s2) -> -Double.compare(s1.getKey(), s2.getKey())).collect(Collectors.toList());
            index = 1;
            for (Map.Entry<Double, List<CGradeSubjectVo>> entry : yxlist2) {
                for(CGradeSubjectVo cgs : entry.getValue()){
                    tjrank.put(key+"_"+cgs.getClassid(),index);
                }
                index = index + entry.getValue().size();
            }

            //批改率排名
            gslist = gslist.stream().sorted((s1, s2) -> -Double.compare(s1.getPgrate(), s2.getPgrate())).collect(Collectors.toList());
            yxlist2 = gslist.stream()
                    .collect(Collectors.groupingBy(CGradeSubjectVo::getPgrate)).entrySet()
                    .stream().sorted((s1, s2) -> -Double.compare(s1.getKey(), s2.getKey())).collect(Collectors.toList());
            index = 1;
            for (Map.Entry<Double, List<CGradeSubjectVo>> entry : yxlist2) {
                for(CGradeSubjectVo cgs : entry.getValue()){
                    pgrank.put(key+"_"+cgs.getClassid(),index);
                }
                index = index + entry.getValue().size();
            }

            //正确率排名
            gslist = gslist.stream().sorted((s1, s2) -> -Double.compare(s1.getZqrate(), s2.getZqrate())).collect(Collectors.toList());
            yxlist2 = gslist.stream()
                    .collect(Collectors.groupingBy(CGradeSubjectVo::getZqrate)).entrySet()
                    .stream().sorted((s1, s2) -> -Double.compare(s1.getKey(), s2.getKey())).collect(Collectors.toList());
            index = 1;
            for (Map.Entry<Double, List<CGradeSubjectVo>> entry : yxlist2) {
                for(CGradeSubjectVo cgs : entry.getValue()){
                    zqrank.put(key+"_"+cgs.getClassid(),index);
                }
                index = index + entry.getValue().size();
            }
        }
        Map<String, List<ReportClasstypeVo>> subjects = parseList(list, 1, now);
        Integer classid = type.getClassid();
        for(Map.Entry s:subjects.entrySet()){
            List<ReportClasstypeVo> slist2 = (List<ReportClasstypeVo>) s.getValue();
            DoubleSummaryStatistics statistics = slist2.stream().mapToDouble(ReportClasstypeVo::getNumrate).summaryStatistics();
            ReportClasstypeVo vo = slist2.get(0);
            if (N_Utils.isEmptyInteger(classid) || classid.equals(vo.getClassid())) {
                try {
                    TReportClassPaper rp = new TReportClassPaper();
                    rp.setClassnum(vo.getClassnum());
                    int fbnum = slist2.stream().mapToInt(ReportClasstypeVo::getFbnum).sum();
                    rp.setPapernum(fbnum);
                    String cgsid = vo.getSchoolid() + "_" + vo.getClasstype() + "_" + vo.getGradeid() + "_" + vo.getSubjectid();
                    if (now) cgsid = cgsid + "_" + vo.getBegindate();
                    ReportRateVo rateVo = maxnum.get(cgsid);
                    rp.setMaxnum(rateVo.getMaxnum());
                    String cgscid = cgsid + "_" + vo.getClassid();
                    rp.setPaperrank(fbrank.get(cgscid));
                    int xsnum = slist2.stream().mapToInt(ReportClasstypeVo::getXsnum).sum();
                    int tjnum = slist2.stream().mapToInt(ReportClasstypeVo::getTjnum).sum();
                    int pgnum = slist2.stream().mapToInt(ReportClasstypeVo::getPgnum).sum();
                    Double pscore = slist2.stream().mapToDouble(ReportClasstypeVo::getPscore).sum();
                    Double uscore = slist2.stream().mapToDouble(ReportClasstypeVo::getUscore).sum();
                    Double commitrate = N_Utils.getIntegerDivideAndMulitiply(tjnum, xsnum);
                    rp.setCommitrate(commitrate);
                    rp.setCommitavgrate(N_Utils.formatDouble(statistics.getAverage(), 2));
                    rp.setCommitmaxrate(rateVo.getTjrate());
                    rp.setCommitraterank(tjrank.get(cgscid));

                    Double checkrate = N_Utils.getIntegerDivideAndMulitiply(pgnum, tjnum);
                    statistics = slist2.stream().mapToDouble(ReportClasstypeVo::getPgrate).summaryStatistics();
                    rp.setCheckrate(checkrate);
                    rp.setCheckavgrate(N_Utils.formatDouble(statistics.getAverage(), 2));
                    rp.setCheckmaxrate(rateVo.getPgrate());
                    rp.setCheckraterank(pgrank.get(cgscid));

                    Double correctrate = N_Utils.getDoubleDivideAndMulitiply(uscore, pscore);
                    statistics = slist2.stream().mapToDouble(ReportClasstypeVo::getZqrate).summaryStatistics();
                    rp.setCorrectrate(correctrate);
                    rp.setCorrectavgrate(N_Utils.formatDouble(statistics.getAverage(), 2));
                    rp.setCorrectmaxrate(rateVo.getZqrate());
                    rp.setCorrectraterank(zqrank.get(cgscid));
                    rp.setCreatetime(N_Utils.getSecondTimestamp());
                    //计算同比
                    TReportClasstype type2 = new TReportClasstype();
                    type2.setReporttype(type.getReporttype());
                    type2.setClassid(vo.getClassid());
                    type2.setSubjectid(vo.getSubjectid());
                    YearOnYearVo y = classReportMapper.getPaperNum(type2);
                    if (y == null) {
                        rp.setPrenumrate(100.0);
                        rp.setCommitprerate(100.0);
                        rp.setCheckprerate(100.0);
                        rp.setCorrectprerate(100.0);
                    } else {
                        rp.setPrenumrate(N_Utils.getIntegerDivideAndMulitiply(fbnum - y.getFbnum(), fbnum));
                        rp.setCommitprerate(N_Utils.formatDouble(commitrate - y.getTjrate(), 2));
                        rp.setCheckprerate(N_Utils.formatDouble(checkrate - y.getPgrate(), 2));
                        rp.setCorrectprerate(N_Utils.formatDouble(correctrate - y.getZqrate(), 2));
                    }
                    if (!now) {
                        Integer creportid = saveType(type, vo);
                        rp.setCreportid(creportid);
                        reportClassPaperMapper.insertSelective(rp);
                    } else {
                        rp.setBegindate(vo.getBegindate());
                        rp.setEnddate(vo.getBegindate());
                    }
                    list1.add(rp);
                } catch (Exception e) {
                    log.error("保存作业数据出错", e.getMessage());
                }
            }
        }
        return list1;
    }

    public List<TReportClassRecite> saveRecite(TReportClasstype type, List<ReportClasstypeVo> list, boolean now) {
        List<TReportClassRecite> list1 = new ArrayList<>();
        Map<String, ReportRateVo> maxnum = new HashMap<>();//年级科目下最高发布次数
        Map<String,Integer> fbrank = new HashMap<>();//年级科目下发布次数排名
        Map<String,Integer> tjrank = new HashMap<>();//年级科目下提交率排名
        Map<String,Integer> zqrank = new HashMap<>();//年级科目下正确率排名
        Map<String, List<ReportClasstypeVo>> gsubjects = parseList(list, 2, now);
        for(Map.Entry s:gsubjects.entrySet()) {
            List<CGradeSubjectVo> gslist = new ArrayList<>();
            List<ReportClasstypeVo> gsubjects2 = (List<ReportClasstypeVo>) s.getValue();
            Map<Integer,List<ReportClasstypeVo>> clist = gsubjects2.stream().collect(Collectors.groupingBy(m->m.getClassid()));
            for(Map.Entry c:clist.entrySet()) {
                List<ReportClasstypeVo> clist2 = (List<ReportClasstypeVo>) c.getValue();
                ReportClasstypeVo vo = clist2.get(0);
                CGradeSubjectVo gs = new CGradeSubjectVo();
                gs.setClassid(vo.getClassid());
                gs.setClasstype(vo.getClasstype());
                gs.setGradeid(vo.getGradeid());
                gs.setSubjectid(vo.getSubjectid());
                gs.setMaxnum(clist2.stream().mapToInt(ReportClasstypeVo::getFbnum).sum());
                int xsum = clist2.stream().mapToInt(ReportClasstypeVo::getXsnum).sum();
                int tjum = clist2.stream().mapToInt(ReportClasstypeVo::getTjnum).sum();
                gs.setNumrate(N_Utils.getIntegerDivideAndMulitiply(tjum,xsum));
                Double pscore = clist2.stream().mapToDouble(ReportClasstypeVo::getPscore).sum();
                Double uscore = clist2.stream().mapToDouble(ReportClasstypeVo::getUscore).sum();
                gs.setZqrate(N_Utils.getDoubleDivideAndMulitiply(uscore,pscore));
                gslist.add(gs);
            }
            ReportRateVo rateVo = new ReportRateVo();
            String key = (String) s.getKey();
            rateVo.setMaxnum(gslist.stream().max(Comparator.comparing(CGradeSubjectVo::getMaxnum)).get().getMaxnum());
            rateVo.setTjrate(gslist.stream().max(Comparator.comparing(CGradeSubjectVo::getNumrate)).get().getNumrate());
            rateVo.setZqrate(gslist.stream().max(Comparator.comparing(CGradeSubjectVo::getZqrate)).get().getZqrate());
            maxnum.put(key,rateVo);

            //发布次数排名
            gslist = gslist.stream().sorted((s1, s2) -> -Long.compare(s1.getMaxnum(), s2.getMaxnum())).collect(Collectors.toList());
            List<Map.Entry<Integer, List<CGradeSubjectVo>>> gslist2 = gslist.stream()
                    .collect(Collectors.groupingBy(CGradeSubjectVo::getMaxnum)).entrySet()
                    .stream().sorted((s1, s2) -> -Long.compare(s1.getKey(), s2.getKey())).collect(Collectors.toList());
            Integer index = 1;
            for (Map.Entry<Integer, List<CGradeSubjectVo>> entry : gslist2) {
                for(CGradeSubjectVo cgs : entry.getValue()){
                    fbrank.put(key+"_"+cgs.getClassid(),index);
                }
                index = index + entry.getValue().size();
            }

            //提交率排名
            gslist = gslist.stream().sorted((s1, s2) -> -Double.compare(s1.getNumrate(), s2.getNumrate())).collect(Collectors.toList());
            List<Map.Entry<Double, List<CGradeSubjectVo>>> yxlist2 = gslist.stream()
                    .collect(Collectors.groupingBy(CGradeSubjectVo::getNumrate)).entrySet()
                    .stream().sorted((s1, s2) -> -Double.compare(s1.getKey(), s2.getKey())).collect(Collectors.toList());
            index = 1;
            for (Map.Entry<Double, List<CGradeSubjectVo>> entry : yxlist2) {
                for(CGradeSubjectVo cgs : entry.getValue()){
                    tjrank.put(key+"_"+cgs.getClassid(),index);
                }
                index = index + entry.getValue().size();
            }

            //正确率排名
            gslist = gslist.stream().sorted((s1, s2) -> -Double.compare(s1.getZqrate(), s2.getZqrate())).collect(Collectors.toList());
            yxlist2 = gslist.stream()
                    .collect(Collectors.groupingBy(CGradeSubjectVo::getZqrate)).entrySet()
                    .stream().sorted((s1, s2) -> -Double.compare(s1.getKey(), s2.getKey())).collect(Collectors.toList());
            index = 1;
            for (Map.Entry<Double, List<CGradeSubjectVo>> entry : yxlist2) {
                for(CGradeSubjectVo cgs : entry.getValue()){
                    zqrank.put(key+"_"+cgs.getClassid(),index);
                }
                index = index + entry.getValue().size();
            }
        }
        Map<String, List<ReportClasstypeVo>> subjects = parseList(list, 1, now);
        Integer classid = type.getClassid();
        for(Map.Entry s:subjects.entrySet()){
            List<ReportClasstypeVo> slist2 = (List<ReportClasstypeVo>) s.getValue();
            DoubleSummaryStatistics statistics = slist2.stream().mapToDouble(ReportClasstypeVo::getNumrate).summaryStatistics();
            ReportClasstypeVo vo = slist2.get(0);
            if (N_Utils.isEmptyInteger(classid) || classid.equals(vo.getClassid())) {
                try {
                    TReportClassRecite rr = new TReportClassRecite();
                    rr.setClassnum(vo.getClassnum());
                    int fbnum = slist2.stream().mapToInt(ReportClasstypeVo::getFbnum).sum();
                    rr.setRecitenum(fbnum);
                    String cgsid = vo.getSchoolid() + "_" + vo.getClasstype() + "_" + vo.getGradeid() + "_" + vo.getSubjectid();
                    if (now) cgsid = cgsid + "_" + vo.getBegindate();
                    ReportRateVo rateVo = maxnum.get(cgsid);
                    rr.setRecitemaxnum(rateVo.getMaxnum());
                    String cgscid = cgsid + "_" + vo.getClassid();
                    rr.setReciterank(fbrank.get(cgscid));
                    int xsnum = slist2.stream().mapToInt(ReportClasstypeVo::getXsnum).sum();
                    int tjnum = slist2.stream().mapToInt(ReportClasstypeVo::getTjnum).sum();
                    Double guiderate = N_Utils.getIntegerDivideAndMulitiply(tjnum, xsnum);
                    rr.setCommitrate(guiderate);
                    rr.setCommitavgrate(N_Utils.formatDouble(statistics.getAverage(), 2));
                    rr.setCommitmaxrate(rateVo.getTjrate());
                    rr.setCommitraterank(tjrank.get(cgscid));

                    Double pscore = slist2.stream().mapToDouble(ReportClasstypeVo::getPscore).sum();
                    Double uscore = slist2.stream().mapToDouble(ReportClasstypeVo::getUscore).sum();
                    Double correctrate = N_Utils.getDoubleDivideAndMulitiply(uscore, pscore);
                    statistics = slist2.stream().mapToDouble(ReportClasstypeVo::getZqrate).summaryStatistics();
                    rr.setCorrectrate(correctrate);
                    rr.setCorrectavgrate(N_Utils.formatDouble(statistics.getAverage(), 2));
                    rr.setCorrectmaxrate(rateVo.getZqrate());
                    rr.setCorrectraterank(zqrank.get(cgscid));
                    rr.setCreatetime(N_Utils.getSecondTimestamp());
                    //计算同比
                    TReportClasstype type2 = new TReportClasstype();
                    type2.setReporttype(type.getReporttype());
                    type2.setClassid(vo.getClassid());
                    type2.setSubjectid(vo.getSubjectid());
                    YearOnYearVo y = classReportMapper.getReciteNum(type2);
                    if (y == null) {
                        rr.setReciteprerate(100.0);
                        rr.setCommitprerate(100.0);
                        rr.setCorrectprerate(100.0);
                    } else {
                        rr.setReciteprerate(N_Utils.getIntegerDivideAndMulitiply(fbnum - y.getFbnum(), fbnum));
                        rr.setCommitprerate(N_Utils.formatDouble(guiderate - y.getTjrate(), 2));
                        rr.setCorrectprerate(N_Utils.formatDouble(correctrate - y.getZqrate(), 2));
                    }
                    if (!now) {
                        Integer creportid = saveType(type, vo);
                        rr.setCreportid(creportid);
                        reportClassReciteMapper.insertSelective(rr);
                    } else {
                        rr.setBegindate(vo.getBegindate());
                        rr.setEnddate(vo.getBegindate());
                    }
                    list1.add(rr);
                } catch (Exception e) {
                    log.error("保存背诵数据出错", e.getMessage());
                }
            }
        }
        return list1;
    }

    public List<TReportClassErrorquefd> saveErrQuestionFd(TReportClasstype type, List<ReportClasstypeVo> list, boolean now) {
        List<TReportClassErrorquefd> list1 = new ArrayList<>();
        Map<String,Integer> maxnum = new HashMap<>();//年级科目下最高发布次数
        Map<String,Double> avgnum = new HashMap<>();//年级科目下平均发布次数
        Map<String,Integer> fbrank = new HashMap<>();//年级科目下发布次数排名
        Map<String, List<ReportClasstypeVo>> gsubjects = parseList(list, 2, now);
        for(Map.Entry s:gsubjects.entrySet()) {
            List<CGradeSubjectVo> gslist = new ArrayList<>();
            List<ReportClasstypeVo> gsubjects2 = (List<ReportClasstypeVo>) s.getValue();
            Map<Integer,List<ReportClasstypeVo>> clist = gsubjects2.stream().collect(Collectors.groupingBy(m->m.getClassid()));
            for(Map.Entry c:clist.entrySet()) {
                List<ReportClasstypeVo> clist2 = (List<ReportClasstypeVo>) c.getValue();
                ReportClasstypeVo vo = clist2.get(0);
                CGradeSubjectVo gs = new CGradeSubjectVo();
                gs.setClassid(vo.getClassid());
                gs.setClasstype(vo.getClasstype());
                gs.setGradeid(vo.getGradeid());
                gs.setSubjectid(vo.getSubjectid());
                gs.setMaxnum(clist2.stream().mapToInt(ReportClasstypeVo::getFbnum).sum());
                gslist.add(gs);
            }
            String key = (String) s.getKey();
            CGradeSubjectVo vo2 = gslist.stream().max(Comparator.comparing(CGradeSubjectVo::getMaxnum)).get();
            maxnum.put(key,vo2.getMaxnum());
            Double anum = gslist.stream().mapToInt(CGradeSubjectVo::getMaxnum).average().getAsDouble();
            anum = N_Utils.formatDouble(anum, 2);
            avgnum.put(key,anum);

            //发布次数排名
            gslist = gslist.stream().sorted((s1, s2) -> -Long.compare(s1.getMaxnum(), s2.getMaxnum())).collect(Collectors.toList());
            List<Map.Entry<Integer, List<CGradeSubjectVo>>> gslist2 = gslist.stream()
                    .collect(Collectors.groupingBy(CGradeSubjectVo::getMaxnum)).entrySet()
                    .stream().sorted((s1, s2) -> -Long.compare(s1.getKey(), s2.getKey())).collect(Collectors.toList());
            Integer index = 1;
            for (Map.Entry<Integer, List<CGradeSubjectVo>> entry : gslist2) {
                for(CGradeSubjectVo cgs : entry.getValue()){
                    fbrank.put(key+"_"+cgs.getClassid(),index);
                }
                index = index + entry.getValue().size();
            }
        }
        Map<String, List<ReportClasstypeVo>> subjects = parseList(list, 1, now);
        Integer classid = type.getClassid();
        for(Map.Entry s:subjects.entrySet()){
            List<ReportClasstypeVo> slist2 = (List<ReportClasstypeVo>) s.getValue();
            ReportClasstypeVo vo = slist2.get(0);
            if (N_Utils.isEmptyInteger(classid) || classid.equals(vo.getClassid())) {
                try {
                    TReportClassErrorquefd ref = new TReportClassErrorquefd();

                    ref.setClassnum(vo.getClassnum());
                    int fbnum = slist2.stream().mapToInt(ReportClasstypeVo::getFbnum).sum();
                    ref.setFdnum(fbnum);
                    String cgsid = vo.getSchoolid() + "_" + vo.getClasstype() + "_" + vo.getGradeid() + "_" + vo.getSubjectid();
                    if (now) cgsid = cgsid + "_" + vo.getBegindate();
                    ref.setFdmaxnum(maxnum.get(cgsid));
                    String cgscid = cgsid + "_" + vo.getClassid();
                    ref.setFdrank(fbrank.get(cgscid));
                    ref.setFdavgnum(avgnum.get(cgsid));
                    ref.setCreatetime(N_Utils.getSecondTimestamp());
                    //计算同比
                    TReportClasstype type2 = new TReportClasstype();
                    type2.setReporttype(type.getReporttype());
                    type2.setClassid(vo.getClassid());
                    type2.setSubjectid(vo.getSubjectid());
                    YearOnYearVo y = classReportMapper.getErrQuestionFdNum(type2);
                    if (y == null) {
                        ref.setPrerate(100.0);
                    } else {
                        ref.setPrerate(N_Utils.getIntegerDivideAndMulitiply(fbnum - y.getFbnum(), fbnum));
                    }
                    if (!now) {
                        Integer creportid = saveType(type, vo);
                        ref.setCreportid(creportid);
                        reportClassErrorquefdMapper.insertSelective(ref);
                    } else {
                        ref.setBegindate(vo.getBegindate());
                        ref.setEnddate(vo.getBegindate());
                    }
                    list1.add(ref);
                } catch (Exception e) {
                    log.error("保存错题辅导数据出错", e.getMessage());
                }
            }
        }
        return list1;
    }

    public List<TReportClassErrorque> saveErrQuestion(TReportClasstype type, List<ReportClasstypeVo> list, boolean now) {
        List<TReportClassErrorque> list1 = new ArrayList<>();
        Map<String,Integer> maxnum = new HashMap<>();//年级科目下最高发布次数
        Map<String,Integer> fbrank = new HashMap<>();//年级科目下发布次数排名
        Map<String, List<ReportClasstypeVo>> gsubjects = parseList(list, 2, false);
        for(Map.Entry s:gsubjects.entrySet()) {
            List<CGradeSubjectVo> gslist = new ArrayList<>();
            List<ReportClasstypeVo> gsubjects2 = (List<ReportClasstypeVo>) s.getValue();
            Map<Integer,List<ReportClasstypeVo>> clist = gsubjects2.stream().collect(Collectors.groupingBy(m->m.getClassid()));
            for(Map.Entry c:clist.entrySet()) {
                List<ReportClasstypeVo> clist2 = (List<ReportClasstypeVo>) c.getValue();
                ReportClasstypeVo vo = clist2.get(0);
                CGradeSubjectVo gs = new CGradeSubjectVo();
                gs.setClassid(vo.getClassid());
                gs.setClasstype(vo.getClasstype());
                gs.setGradeid(vo.getGradeid());
                gs.setSubjectid(vo.getSubjectid());
                gs.setMaxnum(clist2.size());
                gslist.add(gs);
            }
            String key = (String) s.getKey();
            CGradeSubjectVo vo2 = gslist.stream().max(Comparator.comparing(CGradeSubjectVo::getMaxnum)).get();
            maxnum.put(key,vo2.getMaxnum());

            //发布次数排名
            gslist = gslist.stream().sorted((s1, s2) -> -Long.compare(s1.getMaxnum(), s2.getMaxnum())).collect(Collectors.toList());
            List<Map.Entry<Integer, List<CGradeSubjectVo>>> gslist2 = gslist.stream()
                    .collect(Collectors.groupingBy(CGradeSubjectVo::getMaxnum)).entrySet()
                    .stream().sorted((s1, s2) -> -Long.compare(s1.getKey(), s2.getKey())).collect(Collectors.toList());
            Integer index = 1;
            for (Map.Entry<Integer, List<CGradeSubjectVo>> entry : gslist2) {
                for(CGradeSubjectVo cgs : entry.getValue()){
                    fbrank.put(key+"_"+cgs.getClassid(),index);
                }
                index = index + entry.getValue().size();
            }
        }

        Map<String, List<ReportClasstypeVo>> subjects = parseList(list, 1, false);
        Integer classid = type.getClassid();
        for(Map.Entry s:subjects.entrySet()) {
            List<Map> types = new ArrayList<>();
            List<ReportClasstypeVo> slist2 = (List<ReportClasstypeVo>) s.getValue();
            ReportClasstypeVo vo = slist2.get(0);
            if (N_Utils.isEmptyInteger(classid) || classid.equals(vo.getClassid())) {
                try {
                    Map<String, List<ReportClasstypeVo>> qtypes = slist2.stream().collect(Collectors.groupingBy(m -> m.getTypeid()));
                    for (Map.Entry t : qtypes.entrySet()) {
                        List<ReportClasstypeVo> qtypes2 = (List<ReportClasstypeVo>) t.getValue();
                        ReportClasstypeVo t2 = qtypes2.get(0);
                        Map map = new HashMap();
                        map.put("qtypeid", t2.getTypeid());
                        map.put("qtypename", t2.getTypename());
                        map.put("num", qtypes2.size());
                        types.add(map);
                    }
                    vo.setReporttype(type.getReporttype());
                    vo.setBasenum(type.getBasenum());
                    List<Map> points = classReportMapper.listErrQuestionPoint(vo);
                    TReportClassErrorque rq = new TReportClassErrorque();
                    rq.setClassnum(vo.getClassnum());
                    int fbnum = slist2.size();
                    rq.setCeqnum(fbnum);
                    String cgsid = vo.getSchoolid() + "_" + vo.getClasstype() + "_" + vo.getGradeid() + "_" + vo.getSubjectid();
                    rq.setCeqmaxnum(maxnum.get(cgsid));
                    String cgscid = cgsid + "_" + vo.getClassid();
                    rq.setCeqrank(fbrank.get(cgscid));
                    rq.setQtypejson(JSON.toJSONString(types));
                    rq.setPointjson(JSON.toJSONString(points));
                    rq.setCreatetime(N_Utils.getSecondTimestamp());
                    //计算同比
                    TReportClasstype type2 = new TReportClasstype();
                    type2.setReporttype(type.getReporttype());
                    type2.setClassid(vo.getClassid());
                    type2.setSubjectid(vo.getSubjectid());
                    YearOnYearVo y = classReportMapper.getErrQuestionNum(type2);
                    if (y == null) {
                        rq.setPrerate(100.0);
                    } else {
                        rq.setPrerate(N_Utils.getIntegerDivideAndMulitiply(fbnum - y.getFbnum(), fbnum));
                    }
                    if (!now) {
                        Integer creportid = saveType(type, vo);
                        rq.setCreportid(creportid);
                        reportClassErrorqueMapper.insertSelective(rq);
                    } else {
                        rq.setBegindate(type.getBegindate());
                        rq.setEnddate(type.getEnddate());
                    }
                    list1.add(rq);
                } catch (Exception e) {
                    log.error("保存错题数据出错", e.getMessage());
                }
            }
        }
        return list1;
    }

    public List<TReportClassLive> saveLive(TReportClasstype type, List<ReportClasstypeVo> list, boolean now) {
        List<TReportClassLive> list1 = new ArrayList<>();
        Map<String,Integer> maxnum = new HashMap<>();//年级科目下最高发布次数
        Map<String,Integer> zbrank = new HashMap<>();//年级科目下直播次数排名
        Map<String,Integer> scrank = new HashMap<>();//年级科目下时长排名
        Map<String,Integer> minutenum = new HashMap<>();//年级科目下直播时长
        Map<String, List<ReportClasstypeVo>> gsubjects = parseList(list, 2, now);
        for(Map.Entry s:gsubjects.entrySet()) {
            List<CGradeSubjectVo> gslist = new ArrayList<>();
            List<ReportClasstypeVo> gsubjects2 = (List<ReportClasstypeVo>) s.getValue();
            Map<Integer,List<ReportClasstypeVo>> clist = gsubjects2.stream().collect(Collectors.groupingBy(m->m.getClassid()));
            String key = (String) s.getKey();
            for(Map.Entry c:clist.entrySet()) {
                List<ReportClasstypeVo> clist2 = (List<ReportClasstypeVo>) c.getValue();
                ReportClasstypeVo vo = clist2.get(0);
                CGradeSubjectVo gs = new CGradeSubjectVo();
                gs.setClassid(vo.getClassid());
                gs.setClasstype(vo.getClasstype());
                gs.setGradeid(vo.getGradeid());
                gs.setSubjectid(vo.getSubjectid());
                gs.setMaxnum(clist2.stream().mapToInt(ReportClasstypeVo::getFbnum).sum());
                int scnum = clist2.stream().mapToInt(ReportClasstypeVo::getMinutenum).sum();
                gs.setMinutenum(scnum);
                minutenum.put(key+"_"+vo.getClassid(),scnum);
                gslist.add(gs);
            }
            CGradeSubjectVo vo2 = gslist.stream().max(Comparator.comparing(CGradeSubjectVo::getMaxnum)).get();
            maxnum.put(key,vo2.getMaxnum());


            //直播次数排名
            gslist = gslist.stream().sorted((s1, s2) -> -Long.compare(s1.getMaxnum(), s2.getMaxnum())).collect(Collectors.toList());
            List<Map.Entry<Integer, List<CGradeSubjectVo>>> gslist2 = gslist.stream()
                    .collect(Collectors.groupingBy(CGradeSubjectVo::getMaxnum)).entrySet()
                    .stream().sorted((s1, s2) -> -Long.compare(s1.getKey(), s2.getKey())).collect(Collectors.toList());
            Integer index = 1;
            for (Map.Entry<Integer, List<CGradeSubjectVo>> entry : gslist2) {
                for(CGradeSubjectVo cgs : entry.getValue()){
                    zbrank.put(key+"_"+cgs.getClassid(),index);
                }
                index = index + entry.getValue().size();
            }

            //时长排名
            gslist = gslist.stream().sorted((s1, s2) -> -Double.compare(s1.getMinutenum(), s2.getMinutenum())).collect(Collectors.toList());
            List<Map.Entry<Integer, List<CGradeSubjectVo>>> yxlist2 = gslist.stream()
                    .collect(Collectors.groupingBy(CGradeSubjectVo::getMinutenum)).entrySet()
                    .stream().sorted((s1, s2) -> -Double.compare(s1.getKey(), s2.getKey())).collect(Collectors.toList());
            index = 1;
            for (Map.Entry<Integer, List<CGradeSubjectVo>> entry : yxlist2) {
                for(CGradeSubjectVo cgs : entry.getValue()){
                    scrank.put(key+"_"+cgs.getClassid(),index);
                }
                index = index + entry.getValue().size();
            }
        }
        Map<String, List<ReportClasstypeVo>> subjects = parseList(list, 1, now);
        Integer classid = type.getClassid();
        for(Map.Entry s:subjects.entrySet()){
            List<ReportClasstypeVo> slist2 = (List<ReportClasstypeVo>) s.getValue();
            IntSummaryStatistics statistics = slist2.stream().mapToInt(ReportClasstypeVo::getMinutenum).summaryStatistics();
            ReportClasstypeVo vo = slist2.get(0);
            if (N_Utils.isEmptyInteger(classid) || classid.equals(vo.getClassid())) {
                try {
                    TReportClassLive rl = new TReportClassLive();
                    rl.setClassnum(vo.getClassnum());
                    int fbnum = slist2.stream().mapToInt(ReportClasstypeVo::getFbnum).sum();
                    rl.setLivenum(fbnum);
                    String cgsid = vo.getSchoolid() + "_" + vo.getClasstype() + "_" + vo.getGradeid() + "_" + vo.getSubjectid();
                    if (now) cgsid = cgsid + "_" + vo.getBegindate();
                    rl.setLivemaxnum(maxnum.get(cgsid));
                    String cgscid = cgsid + "_" + vo.getClassid();
                    rl.setLiverank(zbrank.get(cgscid));
                    Integer scnum = minutenum.get(cgscid);
                    rl.setMinutenum(scnum);
                    rl.setMinuteave(N_Utils.formatDouble(statistics.getAverage(), 2));
                    rl.setMinutemax(statistics.getMax());
                    rl.setMinuterank(scrank.get(cgscid));
                    rl.setCreatetime(N_Utils.getSecondTimestamp());
                    //计算同比
                    TReportClasstype type2 = new TReportClasstype();
                    type2.setReporttype(type.getReporttype());
                    type2.setClassid(vo.getClassid());
                    type2.setSubjectid(vo.getSubjectid());
                    YearOnYearVo y = classReportMapper.getLiveNum(type2);
                    if (y == null) {
                        rl.setLiveprerate(100.0);
                        rl.setMinutepre(100.0);
                    } else {
                        rl.setLiveprerate(N_Utils.getIntegerDivideAndMulitiply(fbnum - y.getFbnum(), fbnum));
                        rl.setMinutepre(N_Utils.getIntegerDivideAndMulitiply(scnum - y.getTjnum(), scnum));
                    }
                    if (!now) {
                        Integer creportid = saveType(type, vo);
                        rl.setCreportid(creportid);
                        reportClassLiveMapper.insertSelective(rl);
                    } else {
                        rl.setBegindate(vo.getBegindate());
                        rl.setEnddate(vo.getBegindate());
                    }
                    list1.add(rl);
                } catch (Exception e) {
                    log.error("保存直播数据出错", e.getMessage());
                }
            }
        }
        return list1;
    }

    public List<TReportClassGuide> listYx(TReportClasstype type) {
        Integer reporttype = type.getReporttype();
        if (reporttype <= 2) {
            return classReportMapper.listYx(type)
                    .stream().sorted(Comparator.comparing(TReportClassGuide::getCreportid)).collect(Collectors.toList());
        } else {
            //预习
            List<ReportClasstypeVo> yxlist = classReportMapper.listPreview(type);
            type.setReporttype(reporttype - 2);
            setTypeDate(type);
            return savePreview(type, yxlist,true);
        }
    }

    public List<TReportClassRoom> listKt(TReportClasstype type) {
        Integer reporttype = type.getReporttype();
        if (reporttype <= 2) {
            return classReportMapper.listKt(type)
                    .stream().sorted(Comparator.comparing(TReportClassRoom::getCreportid)).collect(Collectors.toList());
        } else {
            //课堂
            List<ReportClasstypeVo> list = classReportMapper.listClassroom(type);
            type.setReporttype(reporttype - 2);
            setTypeDate(type);
            return saveClassroom(type, list, true);
        }
    }

    public List<TReportClassPaper> listZy(TReportClasstype type) {
        Integer reporttype = type.getReporttype();
        if (reporttype <= 2) {
            return classReportMapper.listZy(type)
                    .stream().sorted(Comparator.comparing(TReportClassPaper::getCreportid)).collect(Collectors.toList());
        } else {
            //作业
            List<ReportClasstypeVo> list = classReportMapper.listPaper(type);
            type.setReporttype(reporttype - 2);
            setTypeDate(type);
            return savePaper(type, list, true);
        }
    }

    public List<TReportClassRecite> listBs(TReportClasstype type) {
        Integer reporttype = type.getReporttype();
        if (reporttype <= 2) {
            return classReportMapper.listBs(type)
                    .stream().sorted(Comparator.comparing(TReportClassRecite::getCreportid)).collect(Collectors.toList());
        } else {
            //背诵
            List<ReportClasstypeVo> list = classReportMapper.listRecite(type);
            type.setReporttype(reporttype - 2);
            setTypeDate(type);
            return saveRecite(type, list, true);
        }
    }

    public List<TReportClassErrorquefd> listFd(TReportClasstype type) {
        Integer reporttype = type.getReporttype();
        if (reporttype <= 2) {
            return classReportMapper.listFd(type)
                    .stream().sorted(Comparator.comparing(TReportClassErrorquefd::getCreportid)).collect(Collectors.toList());
        } else {
            //错题辅导
            List<ReportClasstypeVo> list = classReportMapper.listErrQuestionFd(type);
            type.setReporttype(reporttype - 2);
            setTypeDate(type);
            return saveErrQuestionFd(type, list, true);
        }
    }

    public List<TReportClassErrorque> listCt(TReportClasstype type) {
        Integer reporttype = type.getReporttype();
        if (reporttype <= 2) {
            return classReportMapper.listCt(type)
                    .stream().sorted(Comparator.comparing(TReportClassErrorque::getCreportid)).collect(Collectors.toList());
        } else {
            //错题
            List<ReportClasstypeVo> list = classReportMapper.listErrQuestion(type);
            type.setReporttype(reporttype - 2);
            setTypeDate(type);
            return saveErrQuestion(type, list, true);
        }
    }

    public List<TReportClassLive> listZb(TReportClasstype type) {
        Integer reporttype = type.getReporttype();
        if (reporttype <= 2) {
            return classReportMapper.listZb(type)
                    .stream().sorted(Comparator.comparing(TReportClassLive::getCreportid)).collect(Collectors.toList());
        } else {
            //直播
            List<ReportClasstypeVo> list = classReportMapper.listLive(type);
            type.setReporttype(reporttype - 2);
            setTypeDate(type);
            return saveLive(type, list, true);
        }
    }
}
