package com.xhkjedu.sapi.service.system;

import com.xhkjedu.sapi.mapper.system.ModuleMapper;
import com.xhkjedu.sapi.model.system.TModule;
import com.xhkjedu.sapi.utils.ApiUtil;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.sapi.vo.system.ModuleSelVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @ClassName ModuleService
 * Description TODO
 * Author WN
 * Date 2020/6/11 16:16
 **/
@Service
public class ModuleService {
    @Autowired
    private ModuleMapper moduleMapper;

    /**
     *功能描述 添加
     * @author WN
     * @date 2020/6/11
     * @param  * @param module
     * @return java.lang.Integer
     */
    public Integer add(TModule module){
        String id = ApiUtil.getId();
        module.setModuleid(id);
        module.setCreatetime(N_Utils.getSecondTimestamp());
        return moduleMapper.insertSelective(module);
    }

    /**
     * 添加并返回主键
     * @Param [module]
     * @Author ywx
     * @Date 2020/7/2 11:20
     * @return java.lang.Integer
     **/
    public String addAndReturnKey(TModule module){
        String id = ApiUtil.getId();
        module.setModuleid(id);
        module.setCreatetime(N_Utils.getSecondTimestamp());
        moduleMapper.insertSelective(module);
        return id;
    }

    /**
     *功能描述 修改
     * @author WN
     * @date 2020/6/11
     * @param  * @param module
     * @return java.lang.Integer
     */
    public Integer update(TModule module){
        return moduleMapper.updateByPrimaryKeySelective(module);
    }

    /**
     *功能描述  删除
     * @author WN
     * @date 2020/6/11
     * @param  * @param moduleid
     * @return java.lang.Integer
     */
    public Integer del(String moduleid){
        return moduleMapper.deleteByPrimaryKey(moduleid);
    }


    /**
     *功能描述 所有模块
     * @author WN
     * @date 2020/6/11
     * @param  * @param schoolid
     * @param display
     * @return java.util.List<com.xhkjedu.model.system.TModule>
     */
    public List<TModule> listModules(Integer schoolid, Integer moduletype, Integer display){
//        return moduleMapper.listModules(schoolid);
        List<TModule> modules = new ArrayList<>();
        List<TModule> lst = listModulesTree(schoolid,moduletype, display);
        lst.forEach(m->{
            List<TModule> twomodules = m.getModulechilds();
            m.setModulechilds(null);
            modules.add(m);
            twomodules.forEach(m2->{
                List<TModule> threemodules = m2.getModulechilds();
                m2.setModulechilds(null);
                modules.add(m2);
                threemodules.forEach(m3->{
                    modules.add(m3);
                });
            });
        });
        return modules;
    }

    //获取第一层模块
    public List<ModuleSelVo> listLevelModule(Integer schoolid,Integer moduletype){
        return moduleMapper.listLevelModule(schoolid,moduletype);
    }

    /**
     *功能描述  所有模块树形结构
     * @author WN
     * @date 2020/6/15
     * @param  * @param schoolid
     * @param display
     * @return java.util.List<com.xhkjedu.model.system.TModule>
     */
    public List<TModule> listModulesTree(Integer schoolid, Integer moduletype, Integer display){
        List<TModule> lst = moduleMapper.listModules(schoolid,moduletype, display);

        List<TModule> rtnlst = new ArrayList<>();

        for(TModule m : lst){
            if(m.getLevel()==1){
                rtnlst.add(m);
            }
        }

        for(TModule m : rtnlst){
            List<TModule> childs = m.getModulechilds();
            for(TModule mc: lst){
                if(m.getModuleid().equals(mc.getModulepid())){
                    if(!isExistModule(childs,m.getModuleid())){
                        childs.add(mc);
                    }
                }
            }
        }

        return rtnlst;
    }

    //判断集合中是否存在模块
    private boolean isExistModule(List<TModule> lst,String moduleid){
        for(TModule m: lst){
            if(m.getModuleid().equals(moduleid)){
                return true;
            }
        }
        return false;
    }

    /**
     * 模块上移
     * @Param [modules]
     * @Author ywx
     * @Date 2020/8/31 16:24
     * @return void
     **/
    public void moveUp(List<TModule> modules) {
        for (int i = 0; i < modules.size(); i++) {
            TModule module = modules.get(i);
            if (module.getModulepid() == null){
                return;
            }
            if (i == 0) {
                if (module.getMorder() == 1){
                    return;
                }
                moduleMapper.updateOrder(module);
                Integer morder = modules.get(1).getMorder();
                module.setMorder(morder);
            } else {
                module.setMorder(module.getMorder()+1);
            }
        }
        moduleMapper.moveUp(modules);
    }

    //模块选择列表
    public List<Map> listForSel2(Integer schoolid, Integer moduletype) {
        return moduleMapper.listForSel2(schoolid, moduletype);
    }
}
