package com.xhkjedu.sclass.controller.classroom;

import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.xhkjedu.sclass.model.classroom.TClassroom;
import com.xhkjedu.sclass.model.classroom.TClassroomAskstudent;
import com.xhkjedu.sclass.model.classroom.TClassroomAskteacher;
import com.xhkjedu.sclass.model.classroom.TClassroomFile;
import com.xhkjedu.sclass.model.classroom.TClassroomPaper;
import com.xhkjedu.sclass.model.classroom.TClassroomPaperStudent;
import com.xhkjedu.sclass.model.classroom.TClassroomSign;
import com.xhkjedu.sclass.model.paperstudent.TPaperStudent;
import com.xhkjedu.sclass.model.paperstudent.TPaperStudentScantron;
import com.xhkjedu.sclass.service.classroom.ClassroomPaperService;
import com.xhkjedu.sclass.service.classroom.ClassroomService;
import com.xhkjedu.sclass.vo.classroom.ObjectiveVo;
import com.xhkjedu.sclass.vo.classroom.ReportEQuestionVo;
import com.xhkjedu.sclass.vo.classroom.RoomCorrectCallVo;
import com.xhkjedu.sclass.vo.classroom.RoomCorrectExamVo;
import com.xhkjedu.sclass.vo.classroom.RoomCorrectVo;
import com.xhkjedu.sclass.vo.classroom.RoomVo;
import com.xhkjedu.sclass.vo.classroom.SubjectiveVo;
import com.xhkjedu.sclass.vo.paper.CheckClassVo;
import com.xhkjedu.sclass.vo.paperstudent.StudentVo;
import com.xhkjedu.sclass.vo.room.RoomPVo;
import com.xhkjedu.sclass.vo.room.RoomParamVo;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.utils.PageUtil;
import com.xhkjedu.vo.PageResult;
import com.xhkjedu.vo.ResultVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RestController
@RequestMapping("room")
@Slf4j
public class ClassroomController {

    @Resource
    private ClassroomService classroomService;
    @Resource
    private ClassroomPaperService classroomPaperService;

    /**
     * 保存课堂数据
     *
     * @return com.xhkjedu.sclass.vo.ResultVo
     * @Param [params]
     * @Author ywx
     * @Date 2020/7/14 16:45
     **/
    @PostMapping("/saveclass")
    public ResultVo saveClass(@RequestBody List<RoomPVo> rooms) {
        ResultVo resultVo = new ResultVo();
        resultVo.setCode(1);
        try {
            List<Long> savelst = new ArrayList<>();//用于更新课堂时间参与学生
            List<Integer> classlst = new ArrayList<>();//用于更新班级课堂状态
            for (RoomPVo r : rooms) {
                savelst.add(r.getRoomid());
                List<RoomParamVo> params = r.getParams();
                for (RoomParamVo param : params) {
                    String rtype = param.getRtype();
                    String jsonstr = param.getJsonstr();
                    N_Utils.validation(new Object[]{rtype, "请求类型", 2, jsonstr, "请求json", 2});
                    if (rtype.equals("startclass")) {//课堂--开始
                        TClassroom room = JSON.parseObject(jsonstr, TClassroom.class);
                        classlst.add(room.getClassid());
                        boolean rtn = classroomService.startClass(room);//课堂是否重复请求
                        if (rtn) {
                            break;//课堂重复请求结束循环，执行下个课堂数据
                        }
                    } else if (rtype.equals("stopclass")) {//课堂--结束
                        TClassroom room = JSON.parseObject(jsonstr, TClassroom.class);
                        classroomService.stopClass(room);
                    } else if (rtype.equals("savefile")) {//课堂--保存图片
                        TClassroomFile file = JSON.parseObject(jsonstr, TClassroomFile.class);
                        classroomService.saveFile(file);
                    } else if (rtype.equals("askteacher")) {//互动--发起
                        TClassroomAskteacher teacher = JSON.parseObject(jsonstr, TClassroomAskteacher.class);
                        classroomService.saveWork(teacher);
                    } else if (rtype.equals("askstudent")) {//互动--保存学生作答情况
                        TClassroomAskstudent student = JSON.parseObject(jsonstr, TClassroomAskstudent.class);
                        classroomService.saveStuAnswer(student);
                    } else if (rtype.equals("sign")) {//学生--签到
                        TClassroomSign sign = JSON.parseObject(jsonstr, TClassroomSign.class);
                        classroomService.saveStudentSign(sign);
                    } else if (rtype.equals("paper")) {//堂测--发布
                        TClassroomPaper paper = JSON.parseObject(jsonstr, TClassroomPaper.class);
                        classroomPaperService.saveScantronPaper(paper);
                    } else if (rtype.equals("stupaper")) {//堂测--学生作答详情
                        TClassroomPaperStudent student = JSON.parseObject(jsonstr, TClassroomPaperStudent.class);
                        classroomPaperService.saveStudentScantron(student);
                    }
                }
            }

            //最后更新课堂中实际上课学生数量
            classroomService.updateBatchRoomRealStunum(savelst);

            List<Integer> distinctclass = classlst.stream().distinct().collect(Collectors.toList());
            //结束班级
            classroomService.updateBatchRoomState(distinctclass);
            resultVo.setCode(0);
            resultVo.setMsg("保存成功");
        } catch (Exception e) {
            log.error("保存课堂数据失败：" + e.getMessage());
            resultVo.setMsg("保存失败");
        }
        return resultVo;
    }

    @PostMapping("/list_correct")
    public ResultVo listCorrect(@RequestBody RoomVo room) {
        Long roomid = room.getRoomid();
        N_Utils.validation(new Object[]{roomid, "课堂id", 1});
        RoomCorrectVo correct = classroomService.listCorrect(roomid);
        return new ResultVo(0, "获取批阅信息成功", correct);
    }

    @PostMapping("/check_students_call")
    public ResultVo listStudentsByAskTid(@RequestBody RoomCorrectCallVo call) {
        Long asktid = call.getAsktid();
        N_Utils.validation(new Object[]{asktid, "互动id", 1});
        CheckClassVo checkVo = classroomService.listStudentsByAskTid(asktid);
        return new ResultVo(0, "获取提问批改学生列表成功", checkVo);
    }

    @PostMapping("/check_students_exam")
    public ResultVo listStudentsByPaperId(@RequestBody RoomCorrectExamVo exam) {
        Long paperid = exam.getPaperid();
        N_Utils.validation(new Object[]{paperid, "考试id", 1});
        CheckClassVo checkVo = classroomService.listStudentsByPaperId(paperid);
        return new ResultVo(0, "获取考试批改学生列表成功", checkVo);
    }

    @PostMapping("call_objective")
    public ResultVo callObjective(@RequestBody RoomCorrectCallVo call) {
        try {
            Long asktid = call.getAsktid();
            N_Utils.validation(new Object[]{asktid, "互动id", 1});
            ObjectiveVo objective = classroomService.callObjective(asktid, call.getRealstunum());
            return new ResultVo(0, "获取课堂提问客观题详情成功", objective);
        } catch (Exception e) {
            log.error("获取课堂提问客观题详情失败：" + e.getMessage());
            return new ResultVo(1, "获取课堂提问客观题详情失败");
        }
    }

    @PostMapping("exam_objective")
    public ResultVo examObjective(@RequestBody ReportEQuestionVo question) {
        try {
            Long pscaid = question.getPscaid();
            N_Utils.validation(new Object[]{pscaid, "答题卡id", 1});
            ObjectiveVo objective = classroomService.examObjective(pscaid, question.getRealstunum());
            return new ResultVo(0, "获取课堂考试客观题详情成功", objective);
        } catch (Exception e) {
            log.error("获取课堂考试客观题详情失败：" + e.getMessage());
            return new ResultVo(1, "获取课堂考试客观题详情失败");
        }
    }

    @PostMapping("call_subjective")
    public ResultVo callSubjective(@RequestBody RoomCorrectCallVo call) {
        try {
            Long asktid = call.getAsktid();
            N_Utils.validation(new Object[]{asktid, "互动id", 1});
            SubjectiveVo subjective = classroomService.callSubjective(asktid, call.getRealstunum());
            return new ResultVo(0, "获取课堂提问主观题详情成功", subjective);
        } catch (Exception e) {
            log.error("获取课堂提问客观题详情失败：" + e.getMessage());
            return new ResultVo(1, "获取课堂提问主观题详情失败");
        }
    }

    @PostMapping("exam_subjective")
    public ResultVo examSubjective(@RequestBody ReportEQuestionVo question) {
        try {
            Long pscaid = question.getPscaid();
            N_Utils.validation(new Object[]{pscaid, "答题卡id", 1});
            SubjectiveVo subjective = classroomService.examSubjective(pscaid, question.getRealstunum());
            return new ResultVo(0, "获取课堂考试主观题详情成功", subjective);
        } catch (Exception e) {
            log.error("获取课堂考试客观题详情失败：" + e.getMessage());
            return new ResultVo(1, "获取课堂考试主观题详情失败");
        }
    }

    @PostMapping("/correct_call")
    public ResultVo correctCall(@RequestBody StudentVo student) {
        Integer psid = student.getPsid();
        Integer pschecked = student.getPschecked();
        N_Utils.validation(new Object[]{psid, "互动学生id", 1, pschecked, "批改结果", 1});
        classroomService.correctCall(psid, pschecked);
        return new ResultVo(0, "批阅成功");
    }

    @PostMapping("/correct_exam")
    public ResultVo correctExam(@RequestBody TPaperStudent ps) {
        try {
            Integer psid = ps.getPsid();
            Integer checkid = ps.getCreateid();
            String questionResult = ps.getQuestionResult();
            N_Utils.validation(new Object[]{psid, "试卷学生id", 1, checkid, "批改人id", 1});
            int result = classroomPaperService.correctExam(psid, checkid, questionResult);
            if (result > 0) {
                return new ResultVo(0, "批阅成功", 0);
            } else {
                return new ResultVo(0, "批阅完成", 1);
            }
        } catch (Exception e) {
            return new ResultVo(1, "批阅失败");
        }
    }

    @PostMapping("/list_correct_question")
    public ResultVo listCorrectQuestion(@RequestBody TPaperStudent paperStudent) {
        Integer psid = paperStudent.getPsid();
        N_Utils.validation(new Object[]{psid, "考试学生id", 1});
        //根据考试学生id获取考试是否完成批阅
        /*Integer checked = roomPaperService.getCheckedByPsId(psid);
        if (checked == 1){
            return new ResultVo(1,"考试已结束批阅");
        }*/
        List<Map> paperQuestions = classroomPaperService.listCorrectQuestion(psid);
        return new ResultVo(0, "获取考试学生待批改试题列表成功！", paperQuestions);
    }

    @PostMapping("/mark_call")
    public ResultVo markCall(@RequestBody StudentVo student) {
        Integer psid = student.getPsid();
        String studentpic = student.getStudentpic();
        N_Utils.validation(new Object[]{psid, "互动学生id", 1, studentpic, "批改结果", 1});
        classroomService.updateStudentpic(psid, studentpic);
        return new ResultVo(0, "保存成功");
    }

    @PostMapping("/mark_question")
    public ResultVo markQuestion(@RequestBody TPaperStudentScantron ps) {
        Integer pssid = ps.getPssid();
        String useranswer = ps.getUseranswer();
        N_Utils.validation(new Object[]{pssid, "考试学生试题id", 1, useranswer, "批改结果", 2});
        classroomPaperService.updateScantronUseranswer(pssid, useranswer);
        return new ResultVo(0, "保存成功");
    }

    @PostMapping("list_class_stu")
    public ResultVo listStuClass(@RequestBody TClassroom room) {
        try {
            Integer studentid = room.getStudentid();
            String subjectid = room.getSubjectid();
            Integer page = room.getPage();
            Integer pageSize = room.getPageSize();
            N_Utils.validation(new Object[]{studentid, "学生id", 1, subjectid, "科目id", 2
                    , page, "显示页码", 1, pageSize, "显示条数", 1});
            PageHelper.startPage(page, pageSize);
            List<RoomVo> list = classroomService.listStuClass(room);
            PageResult pageResult = PageUtil.getPageResult(new PageInfo<>(list));
            return new ResultVo(0, "获取学生课堂回顾列表成功", pageResult);
        } catch (Exception e) {
            log.error("获取学生课堂回顾列表失败：" + e.getMessage());
            return new ResultVo(1, "获取学生课堂回顾列表失败");
        }
    }

    @PostMapping("/report_stu")
    public ResultVo reportStu(@RequestBody TClassroom room) {
        Long roomid = room.getRoomid();
        Integer createid = room.getCreateid();
        N_Utils.validation(new Object[]{roomid, "课堂id", 1, createid, "学生id", 1});
        return classroomService.getReportStu(roomid, createid);
    }

    /**
     * 获取课堂回顾列表
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [room]
     * @Author ywx
     * @Date 2020/6/15 10:57
     **/
    @PostMapping("list_class_directorid")
    public ResultVo listClassByDirectorId(@RequestBody TClassroom room) {
        try {
            Integer teacherid = room.getTeacherid();
            Integer page = room.getPage();
            Integer pageSize = room.getPageSize();
            N_Utils.validation(new Object[]{teacherid, "教师id", 1, page, "显示页码", 1, pageSize, "显示条数", 1});
            PageHelper.startPage(page, pageSize);
            List<RoomVo> list = classroomService.listClassByDirectorId(room);
            PageResult pageResult = PageUtil.getPageResult(new PageInfo<>(list));
            return new ResultVo(0, "获取课堂回顾列表成功", pageResult);
        } catch (Exception e) {
            log.error("获取课堂回顾列表失败：" + e.getMessage());
            return new ResultVo(1, "获取课堂回顾列表失败");
        }
    }

    /**
     * 课堂回顾详情
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [room]
     * @Author ywx
     * @Date 2020/6/26 9:36
     **/
    @PostMapping("/report")
    public ResultVo report(@RequestBody TClassroom room) {
        N_Utils.validation(new Object[]{room.getRoomid(), "课堂id", 1});
        return classroomService.getReport(room.getRoomid());
    }

    @PostMapping("/del_file")
    public ResultVo delFile(@RequestBody TClassroomFile file) {
        N_Utils.validation(new Object[]{file.getRfid(), "课堂图片id", 1});
        classroomService.delFile(file.getRfid());
        return new ResultVo(0, "删除成功");
    }

    @PostMapping("/list_student")
    public ResultVo listStudent(@RequestBody TClassroom room) {
        N_Utils.validation(new Object[]{room.getClassid(), "班级id", 1, room.getLsbid(), "教材关联id", 2});
        List<Map> list = classroomService.listStudent(room.getClassid(), room.getLsbid());
        return new ResultVo(0, "获取班级学生列表成功", list);
    }

    @PostMapping("/list_estudent")
    public ResultVo listEStudent(@RequestBody TClassroomPaper paper) {
        Long paperid = paper.getPaperid();
        Integer page = paper.getPage();
        Integer pageSize = paper.getPageSize();
        N_Utils.validation(new Object[]{paperid, "考试id", 1, page, "显示页码", 1, pageSize, "显示条数", 1});
        PageHelper.startPage(page, pageSize);
        List<Map> list = classroomPaperService.listEStudent(paperid);
        PageResult pageResult = PageUtil.getPageResult(new PageInfo<>(list));
        return new ResultVo(0, "获取考试按人统计列表成功", pageResult);
    }
}
