package com.xhkjedu.sclass.service.classroom;

import com.xhkjedu.sclass.mapper.classroom.ClassroomActivestudentMapper;
import com.xhkjedu.sclass.mapper.classroom.ClassroomAskstudentMapper;
import com.xhkjedu.sclass.mapper.classroom.ClassroomAskteacherMapper;
import com.xhkjedu.sclass.mapper.classroom.ClassroomFileMapper;
import com.xhkjedu.sclass.mapper.classroom.ClassroomMapper;
import com.xhkjedu.sclass.mapper.classroom.ClassroomPaperFileMapper;
import com.xhkjedu.sclass.mapper.classroom.ClassroomSignMapper;
import com.xhkjedu.sclass.model.classroom.TClassroom;
import com.xhkjedu.sclass.model.classroom.TClassroomAskstudent;
import com.xhkjedu.sclass.model.classroom.TClassroomAskteacher;
import com.xhkjedu.sclass.model.classroom.TClassroomFile;
import com.xhkjedu.sclass.model.classroom.TClassroomPaperFile;
import com.xhkjedu.sclass.model.classroom.TClassroomSign;
import com.xhkjedu.sclass.vo.classroom.ObjectiveVo;
import com.xhkjedu.sclass.vo.classroom.ReportEQuestionVo;
import com.xhkjedu.sclass.vo.classroom.ReportExamVo;
import com.xhkjedu.sclass.vo.classroom.ReportRoomVo;
import com.xhkjedu.sclass.vo.classroom.RoomCorrectCallVo;
import com.xhkjedu.sclass.vo.classroom.RoomCorrectExamVo;
import com.xhkjedu.sclass.vo.classroom.RoomCorrectVo;
import com.xhkjedu.sclass.vo.classroom.RoomVo;
import com.xhkjedu.sclass.vo.classroom.SubjectiveVo;
import com.xhkjedu.sclass.vo.paper.CheckClassVo;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.vo.ResultVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class ClassroomService {
    @Resource
    private ClassroomAskteacherMapper classroomAskteacherMapper;
    @Resource
    private ClassroomActivestudentMapper classroomActivestudentMapper;
    @Resource
    private ClassroomAskstudentMapper classroomAskstudentMapper;
    @Resource
    private ClassroomMapper classroomMapper;
    @Resource
    private ClassroomPaperFileMapper classroomPaperFileMapper;
    @Resource
    private ClassroomSignMapper classroomSignMapper;
    @Resource
    private ClassroomFileMapper classroomFileMapper;

    /**
     * 开始课堂
     *
     * @return void
     * @Param [room]
     * @Author ywx
     * @Date 2020/7/14 17:17
     **/
    @Transactional(rollbackFor = Exception.class)
    public boolean startClass(TClassroom room) {
        boolean rtn = false;//课堂是否重复请求
        try {
            room.setRoomstate(1);
            Integer num = classroomMapper.getStudentNumById(room.getClassid());//根据班级id获取学生数量
            room.setClassnum(num);
            classroomMapper.save(room);
            classroomActivestudentMapper.saveStuActive(room.getRoomid(), room.getClassid(), room.getSchoolid());
        } catch (Exception e) {
            if (e instanceof DuplicateKeyException) {
                rtn = true;
                log.error("接口重复请求，课堂id：" + room.getRoomid() + ",课堂名称：" + room.getRoomname());
            } else {
                log.error("开始课堂失败：" + e.getMessage());
            }
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
        return rtn;
    }

    /**
     * 结束课堂
     *
     * @return void
     * @Param [room]
     * @Author ywx
     * @Date 2020/7/14 17:17
     **/
    @Transactional(rollbackFor = Exception.class)
    public void stopClass(TClassroom room) {
        try {
            Long roomid = room.getRoomid();
            if (N_Utils.isEmptyInteger(room.getEndtime())) {//异常结束：结束时间=开始时间+45分钟
                TClassroom tClassroom = classroomMapper.selectByPrimaryKey(roomid);
                room.setEndtime(tClassroom.getCreatetime() + 60 * 45);
            }
            classroomMapper.endClass(room);//结束课堂
            classroomActivestudentMapper.updateActiveRank(roomid);//更新活跃值排名
        } catch (Exception e) {
            log.error("结束课堂失败：" + e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
    }

    /**
     * 保存学生签到信息
     *
     * @return void
     * @Param [sign]
     * @Author ywx
     * @Date 2020/7/14 16:32
     **/
    @Transactional(rollbackFor = Exception.class)
    public void saveStudentSign(TClassroomSign sign) {
        try {
            Long roomid = sign.getRoomid();
            Integer userid = sign.getUserid();
            List<Integer> studentids = classroomMapper.listStudentId(roomid);//获取班级学生id集合
            if (N_Utils.numInList(userid, studentids)) {//签到学生是本课堂所在班级学生，添加签到信息
                classroomSignMapper.save(sign);
                //如果一个学生退出课堂再进来数据就不对了，最后计算
                //classroomMapper.updateRealStuNum(sign.getRoomid());//更新班级实际签到学生数量
                classroomActivestudentMapper.updateActiveNum(roomid, userid);//更新学生活跃值
            }
        } catch (Exception e) {
            log.error("保存学生签到信息失败：" + e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
    }

    /**
     * 功能描述 最后更新课堂名称
     *
     * @param * @param roomlist
     * @return void
     * @author WN
     * @date 2020/8/14
     */
    public void updateBatchRoomRealStunum(List<Long> roomlist) {
        classroomMapper.updateBatchRoomRealStunum(roomlist);
    }

    /**
     * 互动发起
     *
     * @return void
     * @Param [teacher]
     * @Author ywx
     * @Date 2020/6/15 9:58
     **/
    public void saveWork(TClassroomAskteacher teacher) {
        try {
            if (teacher.getPaperid() == null) {
                teacher.setPaperid(0L);
            }
            if (teacher.getRadionum() == null) {
                teacher.setRadionum(0);
            }
            classroomAskteacherMapper.save(teacher);
        } catch (Exception e) {
            log.error("互动发起失败：" + e.getMessage());
        }
    }

    /**
     * 保存学生作答情况
     *
     * @return void
     * @Param [student]
     * @Author ywx
     * @Date 2020/7/14 16:34
     **/
    @Transactional(rollbackFor = Exception.class)
    public void saveStuAnswer(TClassroomAskstudent student) {
        try {
            if (student.getStudentradio() == null) {
                student.setStudentradio("");
            }
            if (student.getStudentpic() == null) {
                student.setStudentpic("");
            }
            if (student.getRadiocorrect() == null) {
                student.setRadiocorrect(0);
            }
            if (student.getStudentselected() == null) {
                student.setStudentselected(0);
            }
            classroomAskstudentMapper.save(student);
            //保存成功更新学生活跃值
            classroomActivestudentMapper.updateActiveNum(student.getRoomid(), student.getStudentid());
        } catch (Exception e) {
            log.error("保存学生作答情况失败：" + e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
    }

    /**
     * 保存课堂图片
     *
     * @return void
     * @Param [file]
     * @Author ywx
     * @Date 2020/7/14 16:42
     **/
    public void saveFile(TClassroomFile file) {
        try {
            classroomFileMapper.save(file);
        } catch (Exception e) {
            log.error("批量保存课堂图片失败：" + e.getMessage());
        }
    }

    /**
     * 功能描述 批量结束班级之前的课堂
     *
     * @param * @param classidlist
     * @return void
     * @author WN
     * @date 2020/8/14
     */
    public void updateBatchRoomState(List<Integer> classidlist) {
        classroomMapper.updateBatchRoomState(classidlist);
    }

    /**
     * 获取批阅信息
     *
     * @return com.xhkjedu.sclass.vo.classroom.RoomCorrectVo
     * @Param [roomid]
     * @Author ywx
     * @Date 2020/9/9 15:37
     **/
    public RoomCorrectVo listCorrect(Long roomid) {
        RoomCorrectVo correct = classroomMapper.listCorrect(roomid);
        List<RoomCorrectCallVo> calls = correct.getCalls();
        if (N_Utils.isListNotEmpty(calls)) {
            for (RoomCorrectCallVo call : calls) {
                call.setCorrectrate(N_Utils.getIntegerDivideAndMulitiply(call.getCorrectnum(), call.getStunum()));
            }
        }
        List<RoomCorrectExamVo> exams = correct.getExams();
        if (N_Utils.isListNotEmpty(exams)) {
            for (RoomCorrectExamVo exam : exams) {
                exam.setCorrectrate(N_Utils.getIntegerDivideAndMulitiply(exam.getCorrectnum(), exam.getStunum()));
            }
        }
        return correct;
    }

    /**
     * 获取提问批改学生列表
     *
     * @return com.xhkjedu.sclass.vo.paper.CheckClassVo
     * @Param [asktid]
     * @Author ywx
     * @Date 2020/9/9 16:38
     **/
    public CheckClassVo listStudentsByAskTid(Long asktid) {
        CheckClassVo classVo = classroomMapper.listStudentsByAskTid(asktid);
        return getCheckClassVo(classVo);
    }

    /**
     * 计算未提交数、批阅进度
     *
     * @return com.xhkjedu.sclass.vo.paper.CheckClassVo
     * @Param [classVo]
     * @Author ywx
     * @Date 2020/9/10 8:56
     **/
    private CheckClassVo getCheckClassVo(CheckClassVo classVo) {
        Integer ytjnum = classVo.getYtjnum();
        classVo.setWtjnum(classVo.getClassnum() - ytjnum);
        if (!N_Utils.isEmptyInteger(ytjnum)) {
            double scale = N_Utils.getIntegerDivideAndMulitiply(classVo.getYpynum(), ytjnum);
            classVo.setScale(scale);
        } else {
            classVo.setScale(0);
        }
        return classVo;
    }

    /**
     * 获取考试批改学生列表
     *
     * @return com.xhkjedu.sclass.vo.paper.CheckClassVo
     * @Param [paperid]
     * @Author ywx
     * @Date 2020/9/9 17:12
     **/
    public CheckClassVo listStudentsByPaperId(Long paperid) {
        CheckClassVo classVo = classroomMapper.listStudentsByPaperId(paperid);
        List<TClassroomPaperFile> files = classroomPaperFileMapper.listObjsByPaperid(paperid, 3);
        classVo.setFiles(files);
        return getCheckClassVo(classVo);
    }

    /**
     * 获取课堂提问客观题详情
     *
     * @return com.xhkjedu.sclass.vo.classroom.ObjectiveVo
     * @Param [asktid]
     * @Author ywx
     * @Date 2020/9/10 11:12
     **/
    public ObjectiveVo callObjective(Long asktid, Integer realstunum) {
        return classroomMapper.callObjective(asktid, realstunum);
    }

    /**
     * 获取课堂考试客观题详情
     *
     * @return com.xhkjedu.sclass.vo.classroom.ObjectiveVo
     * @Param [pscaid]
     * @Author ywx
     * @Date 2020/9/10 11:21
     **/
    public ObjectiveVo examObjective(Long pscaid, Integer realstunum) {
        return classroomMapper.examObjective(pscaid, realstunum);
    }

    /**
     * 获取课堂提问主客观题详情
     *
     * @return com.xhkjedu.sclass.vo.classroom.SubjectiveVo
     * @Param [asktid]
     * @Author ywx
     * @Date 2020/9/10 14:01
     **/
    public SubjectiveVo callSubjective(Long asktid, Integer realstunum) {
        return classroomMapper.callSubjective(asktid, realstunum);
    }

    /**
     * 获取课堂考试主客观题详情
     *
     * @return com.xhkjedu.sclass.vo.classroom.SubjectiveVo
     * @Param [pscaid]
     * @Author ywx
     * @Date 2020/9/10 14:25
     **/
    public SubjectiveVo examSubjective(Long pscaid, Integer realstunum) {
        return classroomMapper.examSubjective(pscaid, realstunum);
    }

    /**
     * 课堂提问批阅
     *
     * @return void
     * @Param [asksid, radiocorrect]
     * @Author ywx
     * @Date 2020/9/10 15:08
     **/
    public void correctCall(Integer asksid, Integer radiocorrect) {
        classroomMapper.correctCall(asksid, radiocorrect);
    }

    /**
     * 修改学生作答记录（教师添加批注）
     *
     * @return void
     * @Param [asksid, studentpic]
     * @Author ywx
     * @Date 2020/9/11 15:08
     **/
    public void updateStudentpic(Integer asksid, String studentpic) {
        classroomAskstudentMapper.updateStudentpic(asksid, studentpic);
    }

    /**
     * 学生课堂回顾列表
     *
     * @return java.util.List<com.xhkjedu.sclass.vo.classroom.RoomVo>
     * @Param [room]
     * @Author ywx
     * @Date 2020/9/14 16:32
     **/
    public List<RoomVo> listStuClass(TClassroom room) {
        return classroomMapper.listStuClass(room);
    }

    public ResultVo getReportStu(Long roomid, Integer studentid) {
        Map room = classroomMapper.getReportStu(roomid, studentid);
        return new ResultVo(0, "获取学生课堂回顾详情成功", room);
    }

    /**
     * 获取教师指定章节下上过的课堂历史
     *
     * @return java.util.List<com.xhkjedu.sclass.vo.classroom.RoomVo>
     * @Param [room]
     * @Author ywx
     * @Date 2020/6/15 9:19
     **/
    public List<RoomVo> listClassByDirectorId(TClassroom room) {
        return classroomMapper.listClassByDirectorId(room);
    }

    /**
     * 课堂回顾详情
     *
     * @return com.xhkjedu.sclass.vo.ResultVo
     * @Param [roomid]
     * @Author ywx
     * @Date 2020/6/26 9:36
     **/
    public ResultVo getReport(Long roomid) {
        ReportRoomVo room = classroomMapper.getReport(roomid);
        //课堂提问列表
        List<Map> questions = classroomMapper.listQuestionReport(roomid);
        room.setQuestions(questions);

        //课堂考试列表
        List<ReportExamVo> exams = classroomMapper.listPaper(roomid);
        for (ReportExamVo exam : exams) {
            Integer realstunum = exam.getYtjnum();
            exam.setWtjnum(room.getRealstunum()-realstunum);
            List<String> paperfiles = classroomPaperFileMapper.listFilepath(exam.getPaperid());
            exam.setPaperfiles(paperfiles);

            List<ReportEQuestionVo> equestions = classroomMapper.listExamReport(exam.getPaperid(), exam.getClassid());
            for (ReportEQuestionVo equestion : equestions) {
                Double correctrate = N_Utils.getDoubleDivideAndMulitiply(equestion.getCorrectrate(), realstunum);
                equestion.setCorrectrate(correctrate);
                equestion.setCommitrate(N_Utils.getDoubleDivideAndMulitiply(equestion.getCommitrate(), realstunum));
            }
            exam.setQuestions(equestions);
        }
        room.setExams(exams);

        //课堂点名列表
        List<Map> calls = classroomMapper.listCallReport(roomid);
        room.setCalls(calls);

        //课堂抢答列表
        List<Map> asks = classroomMapper.listAskReport(roomid);
        room.setAsks(asks);

        //课堂截图列表
        List<TClassroomFile> files = classroomMapper.listFileReport(roomid);
        room.setFiles(files);

        //课堂学生列表
        List<Map> students = classroomSignMapper.listStudent(roomid);
        room.setStudents(students);
        return new ResultVo(0, "获取课堂回顾详情成功", room);
    }

    //删除课堂图片
    public void delFile(Integer rfid) {
        classroomFileMapper.deleteByPrimaryKey(rfid);
    }

    //班级学生列表
    public List<Map> listStudent(Integer classid, String lsbid) {
        return classroomMapper.listStudent(classid, lsbid);
    }
}
