package com.xhkjedu.sstudy.controller.paperstudent;

import com.xhkjedu.sstudy.model.paperstudent.TPaperStudent;
import com.xhkjedu.sstudy.model.paperstudent.TPaperStudentQuestion;
import com.xhkjedu.sstudy.model.paperstudent.TPaperStudentScantron;
import com.xhkjedu.sstudy.service.paperstudent.PaperStudentService;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.vo.ResultVo;
import com.xhkjedu.sstudy.vo.paperstudent.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * @ClassName PaperStudentController
 * Description 作业--学生
 * Author WN
 * Date 2020/6/9 10:58
 **/
@RestController
@RequestMapping("/ps")
public class PaperStudentController {
    @Autowired
    private PaperStudentService paperStudentService;

    /**
     *功能描述 学生作业列表
     * @date 2020/6/12
     * @param  * @param paperSearch
     */
    @PostMapping("/list")
    public ResultVo list(@RequestBody PaperSearchVo paperSearch) {
        Integer page = paperSearch.getPage();
        Integer pageSize = paperSearch.getPageSize();
        Integer studentid = paperSearch.getStudentid();
        N_Utils.validation(new Object[]{page,"显示页码",1,pageSize,"显示条数",1,studentid,"学生id",1});
        paperSearch.setPage((page - 1)*pageSize);
        List<PaperStudentVo> list = paperStudentService.findAll(paperSearch);
        return new ResultVo(0,"获取列表成功",list);
    }

    /**
     *功能描述 题库作业--开始做作业
     * @date 2020/6/12
     * @param  * @param paperSearch
     */
    @PostMapping("/start_paper")
    public ResultVo startPaper(@RequestBody TPaperStudent paperStudent) {
        Integer psid = paperStudent.getPsid();
        N_Utils.validation(new Object[]{psid,"试卷学生id",1});
        List<PaperStudentQuestionVo> paperQuestionBeans = paperStudentService.startPaper(psid);
        return new ResultVo(0,"开始成功！",paperQuestionBeans);
    }

    /**
     *功能描述 题库作业--学生作答详情
     * @date 2020/6/12
     * @param  * @param paperSearch
     */
    @PostMapping("/paper_question")
    public ResultVo getPaperQuestion(@RequestBody TPaperStudent paperStudent) {
        Integer psid = paperStudent.getPsid();
        N_Utils.validation(new Object[]{psid,"试卷学生id",1});
        List<PaperStudentQuestionVo> paperQuestions = paperStudentService.getPaperQuestion(psid);
        return new ResultVo(0,"获取学生作业对应试题集合根据题号排序成功！",paperQuestions);
    }

    /**
     * 获取学生待批改试题列表
     * @Param [paperStudent]
     * @Author ywx
     * @Date 2020/6/26 13:22
     * @return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/list_correct_question")
    public ResultVo listCorrectQuestion(@RequestBody TPaperStudent paperStudent) {
        Integer psid = paperStudent.getPsid();
        N_Utils.validation(new Object[]{psid,"试卷学生id",1});
        //根据学生作业id获取作业是否完成批阅 2021/8/20 22:05已结束批阅的还可以批阅
        /*Integer checked = paperStudentService.getCheckedByPsId(psid);
        if (checked == 1){
            return new ResultVo(1,"已结束批阅");
        }*/
        List<PaperStudentQuestionVo> paperQuestions = paperStudentService.listCorrectQuestion(psid);
        return new ResultVo(0,"获取学生待批改试题列表成功！",paperQuestions);
    }

    /**
     *功能描述 题库作业--提交单题
     * @date 2020/6/12
     * @param  * @param paperSearch
     */
    @PostMapping("/submit_question")
    public ResultVo submitQuestion(@RequestBody TPaperStudentQuestion psq) {
        N_Utils.validation(new Object[]{psq.getPsqid(),"试卷学生试题id",1});
        int result = 0;
        try {
            result = paperStudentService.submitQuestion(psq);
        } catch (Exception e) {
            new ResultVo(1, "提交失败！");
        }
        if (result >= 0) return new ResultVo(0, "提交成功！");
        else return new ResultVo(1, "提交失败！");
    }

    /**
     *功能描述 题库作业--提交作业
     * @date 2020/6/12
     * @param  * @param paperSearch
     */
    @PostMapping("/submit_paper")
    public ResultVo submitPaper(@RequestBody TPaperStudent ps) {
        N_Utils.validation(new Object[]{ps.getPsid(),"试卷学生id",1});
        int result = 0;
        try {
            result = paperStudentService.submitPaper(ps.getPsid());
        } catch (Exception e) {
            return new ResultVo(1, "提交失败！");
        }
        if (result >= 0) return new ResultVo(0, "提交成功！");
        else return new ResultVo(1, "提交失败！");
    }

    /**
     *功能描述 题库作业--批阅单人作业
     * @date 2020/6/12
     * @param  * @param paperSearch
     */
    @PostMapping("/correct")
    public ResultVo correct(@RequestBody TPaperStudent ps) {
        Integer psid = ps.getPsid();
        Integer checkid = ps.getCreateid();
        String questionResult = ps.getQuestionResult();
        N_Utils.validation(new Object[]{psid,"试卷学生id",1,checkid,"批改人id",1,questionResult,"批改结果",2});
        try {
            paperStudentService.correct(psid , checkid, questionResult);
            return new ResultVo(0, "批改成功");
        } catch (Exception e) {
            return new ResultVo(1, "批改失败");
        }
        /*if (result > 0) return new ResultVo(0, "批改成功！",0);
        else return new ResultVo(0, "批改完成",1);*/
    }

    /**
     * 批阅列表
     * @Param [ps]
     * @Author ywx
     * @Date 2020/6/18 14:35
     * @return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/correct_list")
    public ResultVo correctList(@RequestBody TPaperStudent ps) {
        Integer checkid = ps.getCheckid();
        Integer page = ps.getPage();
        Integer pageSize = ps.getPageSize();
        N_Utils.validation(new Object[]{checkid,"批阅人id",1,page,"显示页码",1,pageSize,"显示条数",1});
        ps.setPage((page - 1)*pageSize);
        List<PaperStudentVo> list = paperStudentService.correctList(ps);
        return new ResultVo(0, "获取学生批阅列表成功！",list);
    }

    /**
     * 题库作业--按题批阅列表
     * @Param [ps]
     * @Author ywx
     * @Date 2021/10/14 14:12
     * @return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/qcorrect_list")
    public ResultVo qcorrectList(@RequestBody TPaperStudent ps) {
        Integer paperid = ps.getPaperid();
        Integer classid = ps.getClassid();
        N_Utils.validation(new Object[]{paperid,"作业id",1,classid,"班级id",1});
        List<Map> map = paperStudentService.qcorrectList(paperid, classid);
        return new ResultVo(0, "获取按题批阅列表成功！",map);
    }

    /**
     * 题库作业--按题批阅
     * @Param [ps]
     * @Author ywx
     * @Date 2021/10/14 14:48
     * @return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/qcorrect")
    public ResultVo qcorrect(@RequestBody TPaperStudentQuestion psq) {
        N_Utils.validation(new Object[]{psq.getPsqid(),"试卷学生试题id",1,psq.getCheckid(),"批改人id",1
                ,psq.getPsid(),"试卷学生id",1});
        try {
            paperStudentService.qcorrect(psq);
            return new ResultVo(0, "批改成功");
        } catch (Exception e) {
            return new ResultVo(1, "批改失败");
        }
    }

    /**
     * 作业说明
     * @Param [ps]
     * @Author ywx
     * @Date 2020/6/18 17:10
     * @return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/paper_comm")
    public ResultVo getPaperComm(@RequestBody TPaperStudent ps) {
        Integer psid = ps.getPsid();
        N_Utils.validation(new Object[]{psid,"作业学生id",1});
        PaperCommVo comm = paperStudentService.getPaperComm(psid);
        return new ResultVo(0, "获取作业说明成功！",comm);
    }

    /**
     *功能描述 保存教师批注
     * @author WN
     * @date 2020/8/24
     * @param  * @param ps
     * @return com.xhkjedu.vo.ResultVo
     */
    @PostMapping("/question_mark")
    public ResultVo userAnswerMark(@RequestBody TPaperStudentQuestion ps) {
        Integer psqid = ps.getPsqid();
        String answers = ps.getUseranswer();
        N_Utils.validation(new Object[]{psqid,"学生试题id",1,answers,"学生作答内容",2});
        paperStudentService.updateUseranswer(answers,psqid);
        return new ResultVo(0, "保存成功");
    }



    //////////////////答题卡（图片作业）start/////////////////////////
    /**
     *功能描述 图片作业--开始做作业
     * @date 2020/6/12
     * @param  * @param paperSearch
     */
    @PostMapping("/sstart_paper")
    public ResultVo scantronStartPaper(@RequestBody TPaperStudent paperStudent) {
        Integer psid = paperStudent.getPsid();
        N_Utils.validation(new Object[]{psid,"试卷学生id",1});
        List<PaperStudentScantronVo> psscanlst = paperStudentService.scantronStartPaper(psid);
        return new ResultVo(0,"开始成功！",psscanlst);
    }

    /**
     *功能描述 图片作业--学生作答详情
     * @date 2020/6/12
     * @param  * @param paperSearch
     */
    @PostMapping("/spaper_question")
    public ResultVo ScantronPaperQuestion(@RequestBody TPaperStudent paperStudent) {
        Integer psid = paperStudent.getPsid();
        N_Utils.validation(new Object[]{psid,"试卷学生id",1});
        List<PaperStudentScantronVo> paperQuestions = paperStudentService.listPaperStudentScantrons(psid);
        return new ResultVo(0,"获取学生作业对应试题集合根据题号排序成功！",paperQuestions);
    }

    @PostMapping("/slist_correct_question")
    public ResultVo sListCorrectQuestion(@RequestBody TPaperStudent paperStudent) {
        Integer psid = paperStudent.getPsid();
        N_Utils.validation(new Object[]{psid,"试卷学生id",1});
        //根据学生作业id获取作业是否完成批阅 2021/8/20 22:05已结束批阅的还可以批阅
        /*Integer checked = paperStudentService.getCheckedByPsId(psid);
        if (checked == 1){
            return new ResultVo(1,"已结束批阅");
        }*/
        List<PaperStudentScantronVo> paperQuestions = paperStudentService.sListCorrectQuestion(psid);
        return new ResultVo(0,"获取学生答题卡待批改试题列表成功！",paperQuestions);
    }

    /**
     *功能描述 图片作业--提交单题
     * @date 2020/6/12
     * @param  * @param paperSearch
     */
    @PostMapping("/ssubmit_question")
    public ResultVo scantronSubmitQuestion(@RequestBody TPaperStudentScantron pss) {
        N_Utils.validation(new Object[]{pss.getPssid(),"试卷学生试题id",1});
        int result = 0;
        try {
            result = paperStudentService.scantronSubmitQuestion(pss);
        } catch (Exception e) {
            return new ResultVo(1, "提交失败！");
        }
        if (result >= 0) return new ResultVo(0, "提交成功！");
        else return new ResultVo(1, "提交失败！");
    }
    /**
     *功能描述 图片作业--提交作业
     * @date 2020/6/12
     * @param  * @param paperSearch
     */
    @PostMapping("/ssubmit_paper")
    public ResultVo scantronSubmitPaper(@RequestBody TPaperStudent ps) {
        N_Utils.validation(new Object[]{ps.getPsid(),"试卷学生id",1});
        int result = 0;
        try {
            result = paperStudentService.scantronSubmitPaper(ps.getPsid());
        } catch (Exception e) {
            return new ResultVo(1, "提交失败！");
        }
        if (result >= 0) return new ResultVo(0, "提交成功！");
        else return new ResultVo(1, "提交失败！");
    }

    /**
     *功能描述 图片作业--批阅单人
     * @date 2020/6/12
     * @param  * @param paperSearch
     */
    @PostMapping("/scorrect")
    public ResultVo scantronCorrect(@RequestBody TPaperStudent ps) {
        Integer psid = ps.getPsid();
        Integer checkid = ps.getCreateid();
        String questionResult = ps.getQuestionResult();
        N_Utils.validation(new Object[]{psid,"试卷学生id",1,checkid,"批改人id",1,questionResult,"批改结果",2});
        try {
            paperStudentService.scantronCorrect(psid , checkid, questionResult);
            return new ResultVo(0, "批改成功");
        } catch (Exception e) {
            return new ResultVo(1, "批改失败");
        }
    }

    /**
     * 附件作业--按题批阅列表
     * @Param [ps]
     * @Author ywx
     * @Date 2021/10/14 14:12
     * @return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/sqcorrect_list")
    public ResultVo sqcorrectList(@RequestBody TPaperStudent ps) {
        Integer paperid = ps.getPaperid();
        Integer classid = ps.getClassid();
        N_Utils.validation(new Object[]{paperid,"作业id",1,classid,"班级id",1});
        List<Map> map = paperStudentService.sqcorrectList(paperid, classid);
        return new ResultVo(0, "获取按题批阅列表成功！",map);
    }

    /**
     * 图片作业--按题批阅
     * @Param [ps]
     * @Author ywx
     * @Date 2021/10/14 14:48
     * @return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/sqcorrect")
    public ResultVo sqcorrect(@RequestBody TPaperStudentScantron pss) {
        N_Utils.validation(new Object[]{pss.getPssid(),"试卷学生试题id",1,pss.getCheckid(),"批改人id",1
                ,pss.getPsid(),"试卷学生id",1});
        try {
            paperStudentService.sqcorrect(pss);
            return new ResultVo(0, "批改成功");
        } catch (Exception e) {
            return new ResultVo(1, "批改失败");
        }
    }

    /**
     *功能描述 保存教师批注
     * @author WN
     * @date 2020/8/24
     * @param  * @param ps
     * @return com.xhkjedu.vo.ResultVo
     */
    @PostMapping("/squestion_mark")
    public ResultVo userAnswerScantronMark(@RequestBody TPaperStudentScantron ps) {
        Integer pssid = ps.getPssid();
        String answers = ps.getUseranswer();
        N_Utils.validation(new Object[]{pssid,"学生试题id",1,answers,"学生作答内容",2});
        paperStudentService.updateScantronUseranswer(answers,pssid);
        return new ResultVo(0, "保存成功");
    }

    //////////////////////////////////答题卡（图片作业）end///////////////////////////////

    //学生作业退回
    @PostMapping("/back_paper")
    public ResultVo backPaper(@RequestBody TPaperStudent ps) {
        N_Utils.validation(new Object[]{ps.getPsid(),"试卷学生id",1});
        return paperStudentService.backPaper(ps.getPsid());
    }

    //获取学生作业状态
    @PostMapping("/paper_state")
    public ResultVo getPaperState(@RequestBody TPaperStudent ps) {
        N_Utils.validation(new Object[]{ps.getPsid(),"试卷学生id",1});
        TPaperStudent paperStudent = paperStudentService.getPaperState(ps.getPsid());
        return new ResultVo(0, "获取学生作业状态成功",paperStudent);
    }
}
