package com.xhkjedu.sstudy.controller.resource;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.xhkjedu.sstudy.model.resource.TGuide;
import com.xhkjedu.sstudy.service.resource.GuideService;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.utils.PageUtil;
import com.xhkjedu.vo.PageResult;
import com.xhkjedu.vo.ResultVo;
import com.xhkjedu.sstudy.vo.paper.AnalyzeSearchVo;
import com.xhkjedu.sstudy.vo.resource.GuideAnalyzeVo;
import com.xhkjedu.sstudy.vo.resource.GuideVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * @ClassName GuideController
 * Description TODO 学生预习
 * Author WN
 * Date 2021/10/20 10:38
 **/
@RequestMapping("/guide")
@RestController
public class GuideController {
    @Autowired
    private GuideService guideService;

    //教师网页端预习列表
    @PostMapping("/teaweb")
    public ResultVo listGuidesForTeacherWeb(@RequestBody TGuide guide) {
        Integer page = guide.getPage();
        Integer pageSize = guide.getPageSize();
        Integer createid = guide.getCreateid();
        String subjectid = guide.getSubjectid();
        N_Utils.validation(new Object[]{page,"显示页码",1,pageSize,"显示条数",1,createid,"教师id",1
                ,subjectid,"科目",2});
        PageHelper.startPage(page, pageSize);
        List<GuideVo> list = guideService.listGuidesForTeacherWeb(guide);
        PageResult pageResult = PageUtil.getPageResult(new PageInfo<>(list));
        return new ResultVo(0,"获取教师预习列表成功",pageResult);
    }
    //教师app预习列表
    @PostMapping("/teaapp")
    public ResultVo listGuidesForTeacherApp(@RequestBody TGuide guide) {
        Integer page = guide.getPage();
        Integer pageSize = guide.getPageSize();
        Integer createid = guide.getCreateid();
        String subjectid = guide.getSubjectid();
        N_Utils.validation(new Object[]{page,"显示页码",1,pageSize,"显示条数",1,createid,"教师id",1
                ,subjectid,"科目",2});
        guide.setPage((page - 1)*pageSize);
        List<Map> list = guideService.listGuidesForTeacherApp(guide);
        return new ResultVo(0,"获取教师预习列表成功",list);
    }
    //学生网页端预习列表
    @PostMapping("/stuweb")
    public ResultVo listGuidesForStudentWeb(@RequestBody TGuide guide) {
        Integer page = guide.getPage();
        Integer pageSize = guide.getPageSize();
        Integer studentid = guide.getStudentid();
        N_Utils.validation(new Object[]{page,"显示页码",1,pageSize,"显示条数",1,studentid,"学生id",1});
        PageHelper.startPage(page, pageSize);
        List<GuideVo> list = guideService.listGuidesForStuWeb(guide);
        PageResult pageResult = PageUtil.getPageResult(new PageInfo<>(list));
        return new ResultVo(0,"获取学生预习列表成功",pageResult);
    }
    //学生APP预习列表
    @PostMapping("/stuapp")
    public ResultVo listGuidesForStuApp(@RequestBody TGuide guide) {
        Integer page = guide.getPage();
        Integer pageSize = guide.getPageSize();
        Integer studentid = guide.getStudentid();
        String subjectid = guide.getSubjectid();
        N_Utils.validation(new Object[]{page,"显示页码",1,pageSize,"显示条数",1,studentid,"学生id",1
                ,subjectid,"科目",2});
        guide.setPage((page - 1)*pageSize);
        List<Map> list = guideService.listGuidesForStuApp(guide);
        return new ResultVo(0,"获取学生预习列表成功",list);
    }

    //详情
    @PostMapping("/detail")
    public ResultVo detail(@RequestBody TGuide guide) {
        N_Utils.validation(new Object[]{guide.getGuideid(),"预习id",1});

        GuideVo guideVo = guideService.getGuideByGuideid(guide.getGuideid());
        return new ResultVo(0, "获取预习详情成功",guideVo);
    }

    //保存预习
    @PostMapping("/add")
    public ResultVo add(@RequestBody TGuide guide) {
        if (CollectionUtils.isEmpty(guide.getGuideGroups())){
            return new ResultVo(1,"学生预习班级不能为空");
        }
        Integer createid = guide.getCreateid();
        String subjectid = guide.getSubjectid();
        String lsbid = guide.getLsbid();
        N_Utils.validation(new Object[]{createid,"教师id",1,subjectid,"科目",2,lsbid,"教材",2});

        guideService.save(guide);
        return new ResultVo(0, "保存成功");
    }

    //分享预习
    @PostMapping("/share")
    public ResultVo shareGuide(@RequestBody TGuide guide) {
        if (CollectionUtils.isEmpty(guide.getGuideGroups())){
            return new ResultVo(1,"学生预习班级不能为空");
        }
        String resourceid = guide.getResourceid();
        Integer createid = guide.getCreateid();
        String subjectid = guide.getSubjectid();
        String lsbid = guide.getLsbid();
        N_Utils.validation(new Object[]{resourceid,"资源",2,createid,"教师id",1,subjectid,"科目",2,lsbid,"教材",2});

        guideService.shareGuide(guide);
        return new ResultVo(0, "分享成功");
    }

    //删除预习
    @PostMapping("/del")
    public ResultVo delete(@RequestBody TGuide guide) {
        N_Utils.validation(new Object[]{guide.getGuideid(),"预习id",1});

        guideService.delete(guide.getGuideid());
        return new ResultVo(0, "删除成功");
    }

    //////统计相关//////

    @PostMapping("/preview_analyze")
    public ResultVo previewAnalyze(@RequestBody AnalyzeSearchVo paper) {
        if (paper.getUsertype() == null) return new ResultVo(1,"用户类型不能为空");
        Integer schoolid = paper.getSchoolid();
        Integer gradeid = paper.getGradeid();
        N_Utils.validation(new Object[]{schoolid,"学校id",1,gradeid,"年级id",1});
        List<GuideAnalyzeVo> list = guideService.getPreviewAnalyze(paper);
        return new ResultVo(0,"获取年级下学生预习统计列表成功",list);
    }

    @PostMapping("/preview_list")
    public ResultVo listPreview(@RequestBody AnalyzeSearchVo paper) {
        Integer page = paper.getPage();
        Integer pageSize = paper.getPageSize();
        Integer teacherid = paper.getTeacherid();
        Integer gradeid = paper.getGradeid();
        N_Utils.validation(new Object[]{page,"显示页码",1,pageSize,"显示条数",1,teacherid,"教师id",1,gradeid,"年级id",1});
        guideService.setYear(paper);
        guideService.setClassIds(paper);
        PageHelper.startPage(page, pageSize);
        List<Map> list = guideService.listPreview(paper);
        PageResult pageResult = PageUtil.getPageResult(new PageInfo(list));
        return new ResultVo(0,"获取年级下教师发布学生预习列表成功",pageResult);
    }

    @PostMapping("/preview_stu")
    public ResultVo previewStu(@RequestBody AnalyzeSearchVo paper){
        if (paper.getUsertype() == null) return new ResultVo(1,"用户类型不能为空");
        Integer schoolid = paper.getSchoolid();
        Integer gradeid = paper.getGradeid();
        Integer page = paper.getPage();
        Integer pageSize = paper.getPageSize();
        N_Utils.validation(new Object[]{schoolid,"学校id",1,gradeid,"年级id",1
                ,page,"显示页码",1,pageSize,"显示条数",1});
        guideService.setYear(paper);
        guideService.setClassIds(paper);
        PageHelper.startPage(page, pageSize);
        List<Map> list = guideService.listPreviewStuGrade(paper);
        PageResult pageResult = PageUtil.getPageResult(new PageInfo(list));
        return new ResultVo(0,"获取预习统计--按学生列表成功",pageResult);
    }

    @PostMapping("/preview_sdetail")
    public ResultVo listPreviewStuDetail(@RequestBody AnalyzeSearchVo paper) {
        if (paper.getUsertype() == null) return new ResultVo(1,"用户类型不能为空");
        Integer studentid = paper.getStudentid();
        Integer gradeid = paper.getGradeid();
        N_Utils.validation(new Object[]{studentid,"学生id",1,gradeid,"年级id",1});
        guideService.setYear(paper);
        Map map = guideService.listPreviewStuDetail(paper);
        return new ResultVo(0,"获取预习统计--按学生详情成功",map);
    }

    @PostMapping("/preview_sexport")
    public ResultVo listPaperStuExport(@RequestBody AnalyzeSearchVo paper) {
        if (paper.getUsertype() == null) return new ResultVo(1,"用户类型不能为空");
        Integer schoolid = paper.getSchoolid();
        Integer gradeid = paper.getGradeid();
        N_Utils.validation(new Object[]{schoolid,"学校id",1,gradeid,"年级id",1});
        Map list = guideService.listPreviewStuExport(paper);
        return new ResultVo(0,"获取预习统计--按学生导出列表成功",list);
    }
}
