package com.xhkjedu.sstudy.service.paperstudent;

import com.xhkjedu.sstudy.mapper.gradeclass.ClassMapper;
import com.xhkjedu.sstudy.mapper.paper.PaperMapper;
import com.xhkjedu.sstudy.mapper.paper.PaperQtypeQuestionMapper;
import com.xhkjedu.sstudy.mapper.paper.PaperScantronMapper;
import com.xhkjedu.sstudy.mapper.paperstudent.PaperGroupMapper;
import com.xhkjedu.sstudy.mapper.paperstudent.PaperStudentMapper;
import com.xhkjedu.sstudy.model.paper.TPaper;
import com.xhkjedu.sstudy.model.paperstudent.TPaperGroup;
import com.xhkjedu.sstudy.model.paperstudent.TPaperStudent;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.vo.ResultVo;
import com.xhkjedu.sstudy.vo.paperstudent.PaperGroupParam;
import com.xhkjedu.sstudy.vo.paperstudent.PaperGroupVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName PaperGroupService
 * Description 作业--班级
 * Author WN
 * Date 2020/6/9 11:03
 **/
@Slf4j
@Service
public class PaperGroupService {
    @Autowired
    private PaperGroupMapper paperGroupMapper;
    @Autowired
    private ClassMapper classMapper;
    @Autowired
    private PaperMapper paperMapper;
    @Autowired
    private PaperStudentMapper paperStudentMapper;

    @Autowired
    private PaperQtypeQuestionMapper paperQtypeQuestionMapper;
    @Autowired
    private PaperScantronMapper paperScantronMapper;

    /**
     *功能描述
     * @author WN
     * @date 2020/6/9
     * @param  * @param paperGroupVo
     * @return void
     */
    @Transactional(rollbackFor = Exception.class)
    public ResultVo savePaperGroup(PaperGroupParam paperGroupParam){
        ResultVo resultVo = null;
        try {
            //获取试卷详细信息
            TPaper paper = paperMapper.selectByPrimaryKey(paperGroupParam.getPaperid());
            if (paper.getPaperstate() == 22) return new ResultVo(0,"保存成功");
            //班级
            List<TPaperGroup> classlst = paperGroupParam.getClasslst();

            if(classlst.size()==0){
                return new ResultVo(1,"发布作业未选择班级");
            }else{

                //保存作业分组、作业学生
                resultVo = this.savePaperStudent(paperGroupParam,classlst,paper);
                if(resultVo.getCode()==1){
                    return resultVo;
                }else{
                    //获取作业是否全是非主观题
                    int count = 0;
                    if(paper.getPapertype()==1){
                        //题库作业
                        count = paperQtypeQuestionMapper.getQuestionCountCtype(paperGroupParam.getPaperid());
                    }else{
                        //答题卡作业
                        count = paperScantronMapper.getQuestionCountCtype(paperGroupParam.getPaperid());
                    }

                    //保存作业分组、作业学生成功后，修改作业状态
                    paperMapper.updatePaperState(22,paperGroupParam.getPaperid(),count);

                    resultVo.setObj(count);//如果count=0则，代表全部是选择题
                }
            }

        }catch (Exception e){
            log.error("发布作业失败：" + e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new ResultVo(1,"发布失败");
        }
        return resultVo;
    }

    private ResultVo savePaperStudent(PaperGroupParam paperGroupParam, List<TPaperGroup> classlst, TPaper paper){
        List<TPaperGroup> savegroup = new ArrayList<>();//班级分组
        List<TPaperStudent> saveps = new ArrayList<>();//班级分组学生

        for (TPaperGroup pg:classlst){
            List<Integer> groups = pg.getGroups();
            if(groups==null || groups.size()==0){
                return new ResultVo(1,"发布作业未选择分组");
            }else{
                pg.setPaperid(paperGroupParam.getPaperid());
                pg.setPgstate(22);//布置作业
                pg.setStarttime(paperGroupParam.getStarttime());
                pg.setStoptime(paperGroupParam.getStoptime());
                pg.setShowtype(paperGroupParam.getShowtype());
                pg.setCreateid(paperGroupParam.getCreateid());
                pg.setSchoolid(paperGroupParam.getSchoolid());
                pg.setCreatetime(N_Utils.getSecondTimestamp());
                pg.setChecked(0);


                savegroup.add(pg);
                String groupids = StringUtils.join(groups,",");
                //获取选中班级分组下所有学生
                List<Integer> stus = classMapper.listStudentidsByGroupids(groupids);

                for(Integer stu:stus){
                    TPaperStudent ps = new TPaperStudent();
                    ps.setPaperid(paperGroupParam.getPaperid());
                    ps.setClassid(pg.getClassid());
                    ps.setStudentid(stu);
                    ps.setPapernum(paper.getPapernum());
                    ps.setPaperscore(paper.getPaperscore());
                    ps.setCreateid(paperGroupParam.getCreateid());
                    ps.setSchoolid(paperGroupParam.getSchoolid());
                    ps.setCreatetime(N_Utils.getSecondTimestamp());

                    saveps.add(ps);
                }
            }
        }

        //批量保存作业班级，作业班级学生信息
        try {
            paperGroupMapper.insertList(savegroup);
        } catch (Exception e) {
            if (e instanceof DuplicateKeyException) {
                return new ResultVo(0,"保存成功");
            }
        }
        paperStudentMapper.saveBatchPaperStudent(saveps);
        return new ResultVo(0,"保存成功");
    }


    /**
     * 获取参与作业学生列表
     * @Param [paperid]
     * @Author ywx
     * @Date 2020/7/24 14:52
     * @return java.util.List<com.xhkjedu.vo.paperstudent.PaperGroupVo>
     **/
    public List<PaperGroupVo> listPaperStudent(Integer paperid) {
        return paperGroupMapper.listPaperStudent(paperid);
    }
}
