package com.xhkjedu.sstudy.service.paperstudent;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.sstudy.mapper.paper.PaperScantronMapper;
import com.xhkjedu.sstudy.mapper.paperstudent.PaperGroupMapper;
import com.xhkjedu.sstudy.mapper.paperstudent.PaperStudentMapper;
import com.xhkjedu.sstudy.mapper.question.QuestionMapper;
import com.xhkjedu.sstudy.model.paper.TPaperScantron;
import com.xhkjedu.sstudy.model.paperstudent.TPaperGroup;
import com.xhkjedu.sstudy.model.paperstudent.TPaperStudent;
import com.xhkjedu.sstudy.model.paperstudent.TPaperStudentQuestion;
import com.xhkjedu.sstudy.model.paperstudent.TPaperStudentScantron;
import com.xhkjedu.sstudy.model.question.TQuestion;
import com.xhkjedu.sstudy.service.question.QuestionClassService;
import com.xhkjedu.sstudy.utils.ArithUtil;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.vo.ResultVo;
import com.xhkjedu.sstudy.vo.paperstudent.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @author ywx
 * @classname PaperStudentService
 * @description 
 * @date 2020/6/10 10:50
 **/
@Service
@Slf4j
public class PaperStudentService {
    @Autowired
    private PaperStudentMapper paperStudentMapper;
    @Autowired
    private PaperStudentQuestionService paperStudentQuestionService;
    @Resource
    private QuestionMapper questionMapper;
    @Resource
    private QuestionClassService questionClassService;
    @Resource
    private PaperGroupMapper paperGroupMapper;

    @Autowired
    private PaperStudentScantronService paperStudentScantronService;
    @Autowired
    private PaperScantronMapper paperScantronMapper;

    private static Integer[] QTYPE = {1,2,4,5,6,7,8,10};

    //获取详情
    public TPaperStudent findById(Integer psid) {
        return paperStudentMapper.findById(psid);
    }

    //获取列表
    public List <PaperStudentVo> findAll(PaperSearchVo paperSearch) {
        return paperStudentMapper.findAll(paperSearch);
    }

    /**
     * 学生开始做作业
     * @Param [psid]
     * @Author ywx
     * @Date 2020/6/10 15:56
     * @return java.lang.Integer
     **/
    public synchronized List<PaperStudentQuestionVo> startPaper(Integer psid) {
        updateState(null,null,psid,1);
        List<PaperStudentQuestionVo> paperQuestionBeans = getPaperQuestion(psid);
        if (paperQuestionBeans.size() <= 0){
            paperStudentQuestionService.batchSave(psid);
            paperQuestionBeans = getPaperQuestion(psid);
        }
        return paperQuestionBeans;
    }

    /**
     * 获取学生作业详情
     * @Param [psid]
     * @Author ywx
     * @Date 2020/6/10 15:52
     * @return java.util.List<com.xhkjedu.vo.paperstudent.PaperStudentQuestionVo>
     **/
    public List<PaperStudentQuestionVo> getPaperQuestion(Integer psid) {
        return paperStudentQuestionService.getPaperQuestion(psid);
    }

    /**
     * 获取学生待批改试题列表
     * @Param [psid]
     * @Author ywx
     * @Date 2020/6/26 13:21
     * @return java.util.List<com.xhkjedu.vo.paperstudent.PaperStudentQuestionVo>
     **/
    public List<PaperStudentQuestionVo> listCorrectQuestion(Integer psid) {
        return paperStudentQuestionService.listCorrectQuestion(psid);
    }

    /**
     * 更新学生作业提交、批改状态
     * @Param [psstate, pschecked, psid, operate]
     * @Author ywx
     * @Date 2020/6/10 15:54
     * @return java.lang.Integer
     **/
    public Integer updateState(Integer psstate, Integer pschecked, Integer psid, Integer operate){
        return paperStudentMapper.updateState(psstate, pschecked, psid, operate);
    }

    public int submitQuestion(TPaperStudentQuestion psq) throws Exception {
        try {
            //获取作业学生试题详情
            TPaperStudentQuestion psq1 = paperStudentQuestionService.findById(psq.getPsqid());
            //获取试题详情
            TQuestion questionBean = questionMapper.selectByPrimaryKey(psq1.getQuestionid());
            psq.setChecked(0);
            psq.setAnswertime(N_Utils.getSecondTimestamp());
            //判断是否作答1.提交但未作答设置零分，2.作答设置答题状态为2，3.选择题提交为空判错入错题库
            String useranswer = psq.getUseranswer();
            List<String> userAnswers = JSON.parseArray(useranswer, String.class);
            String qanswer = questionBean.getQanswer();
            List<String> qanswers = JSON.parseArray(qanswer, String.class);
            //计算用户得分
            Double userScore = 0.0;
            //设置保留位数
            DecimalFormat df=new DecimalFormat("0.00");
            if (N_Utils.isListEmpty(userAnswers) && N_Utils.isEmpty(psq.getUseranswertext())
                    && N_Utils.isEmpty(psq.getUseranswertxt())) {
                psq.setStuscore(userScore);
                psq.setChecked(1);
                psq.setCheckid(0);
                psq.setChecktime(N_Utils.getSecondTimestamp());
                psq.setAnswered(0);
            } else {
                psq.setAnswered(1);
                //自动批改判断得分
                Integer ctype = questionBean.getCtype();
                if (Arrays.asList(QTYPE).contains(ctype)) {
                    if (ctype == 1) {//单选
                        if (useranswer.equalsIgnoreCase(qanswer)) {
                            userScore = psq1.getQscore();
                        }
                    } else if (ctype == 2) {//多选
                        //多选不得分,或者选错不得分，不用处理
                        if (userAnswers.get(0).length() <= questionBean.getQanswer().length()){
                            String[] qanswerList = qanswers.get(0).split("");
                            String[] userAnswerList = userAnswers.get(0).split("");
                            int rightNum = 0;//选对个数
                            for (String userAnswer : userAnswerList){
                                int right = 0;
                                for (String answer : qanswerList){
                                    if (answer.equals(userAnswer)){
                                        right = 1;
                                        rightNum += 1;
                                    }
                                }
                                if (right == 0){//选错直接0分，结束循环
                                    rightNum = 0;
                                    break;
                                }
                            }
                            if (rightNum > 0){//选对计算得分
                                if (qanswerList.length == rightNum) {//1.全选对满分
                                    userScore = psq1.getQscore();
                                } else if (qanswerList.length > rightNum) {//2.少选得一半分
                                    userScore = ArithUtil.div(psq1.getQscore(), 2, 1);
                                }
                            }
                        }
                    } else if (ctype == 4 || ctype == 5 || ctype == 6) {
                        //判断题
                        if (useranswer.equalsIgnoreCase(qanswer)) {
                            userScore = psq1.getQscore();
                        }
                    } else if (ctype == 7 || ctype == 8 || ctype == 10) {//完型填空、阅读理解、任务型阅读
                        //计算每小题分数
                        Double qscore = ArithUtil.div(psq1.getQscore(),qanswers.size());
                        Double userScoreDouble = 0D;
                        for (int i = 0;i<qanswers.size();i++) {
                            if (qanswers.get(i).equalsIgnoreCase(userAnswers.get(i))) {
                                userScoreDouble = ArithUtil.add(userScoreDouble, qscore);
                            }
                        }
                        userScore = ArithUtil.round(userScoreDouble,1);
                    }
                    if (userScore.compareTo(psq1.getQscore()) == 1){
                        userScore = psq1.getQscore();
                    }
                    psq.setStuscore(userScore);
                    psq.setChecked(1);
                    psq.setCheckid(0);
                    psq.setChecktime(N_Utils.getSecondTimestamp());
                }
            }
            Integer costtime = psq1.getCosttime();
            if (costtime == null) costtime = 0;
            psq.setCosttime(psq.getCosttime()+costtime);
            return paperStudentQuestionService.update(psq);
//            return updateStuScore(psq1.getPsid());//自动批改单题提交时修改学生总得分
        } catch (Exception e) {
            log.error("提交试题失败："+e.getMessage());
            throw new Exception("提交试题失败");
        }
    }

    /**
     * 批改完修改学生总得分和批改状态 (结构性试题)
     * @Param [psid]
     * @Author ywx
     * @Date 2020/6/10 16:28
     * @return java.lang.Integer
     **/
    public Integer updateStuScore(Integer psid, Integer checkid){
        return paperStudentMapper.updateStuScore(psid,checkid);
    }

    /**
     * 提交作业
     * @Param [psid]
     * @Author ywx
     * @Date 2020/6/11 10:44
     * @return int
     **/
    public int submitPaper(Integer psid) throws Exception {
        try {
            TPaperStudent ps = findById(psid);
            //如果学生作业不存在或是已提交，直接返回1提交成功
            if (ps == null || ps.getPsstate() != 0 ) return 1;
            Integer paperid = ps.getPaperid();
            Integer classid = ps.getClassid();
            TPaperStudent paperStudentBean = new TPaperStudent();
            paperStudentBean.setPsid(psid);
            int nowTime = N_Utils.getSecondTimestamp();
            TPaperGroup pg = paperGroupMapper.selectByPaperAndClassId(paperid, classid);
            if (pg.getStoptime() < nowTime) paperStudentBean.setPsstate(2);
            else paperStudentBean.setPsstate(1);

            paperStudentBean.setStoptime(nowTime);
            int costtime = paperStudentQuestionService.getSumQuesitonsCosttime(psid);
            paperStudentBean.setCosttime(costtime);

            //答案显示方式：1作业截至时间，2提交后显示
            Integer showanswer = 0;
            if (pg.getShowtype() == 2){
                showanswer = 1;
            }
            paperStudentBean.setShowanswer(showanswer);
            Integer result = updatePaper(psid, paperStudentBean, pg);

            return result;
        } catch (Exception e) {
            log.error("提交作业失败："+e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            throw new Exception("提交失败");
        }
    }

    /**
     * 题库作业提交更新操作
     * @Param [psid, paperStudentBean, pg]
     * @Author ywx
     * @Date 2020/11/7 10:00
     * @return java.lang.Integer
     **/
    @Transactional(rollbackFor = Exception.class)
    public Integer updatePaper(Integer psid, TPaperStudent paperStudentBean, TPaperGroup pg) throws Exception {
        Integer result = null;
        try {
            //把学生未做的题设置分值为0，状态为已批改
            paperStudentQuestionService.updatePSQByNotDo(psid);
            int psqStudentCount = paperStudentQuestionService.getNotCorrect(psid);
            Integer pschecked = 0;
            if (psqStudentCount <= 0) {//该学生作业已经全部批改完成
                //作业学生关联表，学生批改状态置为已批改
                pschecked = 1;
            }
            paperStudentBean.setPschecked(pschecked);
            result = paperStudentMapper.updatePaper(paperStudentBean);
            questionClassService.submitPaper(pg);//自动批改的作业全部批改完调用批阅结束方法
        } catch (Exception e) {
            log.error("提交作业失败："+e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            throw new Exception("提交失败");
        }
        return result;
    }

    //题库作业--批阅单人作业
    public void correct(Integer psid, Integer checkid, String questionResult) throws Exception {
        try {
            List<TPaperStudentQuestion> psqs = JSON.parseArray(questionResult,TPaperStudentQuestion.class);
            for (TPaperStudentQuestion psq : psqs) {
                //设置批改状态为已批改
                psq.setCheckid(checkid);
                psq.setChecked(1);
                psq.setChecktime(N_Utils.getSecondTimestamp());
            }
            updateCorrect(psid, checkid, psqs);
            /*TPaperStudent ps = findById(psid);
            Integer paperid = ps.getPaperid();
            Integer classid = ps.getClassid();
            //判断该作业该班级时候已经批改完成
            //查询如果为0，则证明本作业本班级截止当前时间已经提交作业学生全部批改完成
            //Integer notCorrect = paperStudentMapper.getNotCorrect(paperid, classid);
            return notCorrect;*/
        } catch (Exception e) {
            log.error("批改作业失败："+e.getMessage());
            throw new Exception("批改失败");
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void updateCorrect(Integer psid, Integer checkid, List<TPaperStudentQuestion> psqs) throws Exception {
        try {
            //批量更新试题批改信息
            paperStudentMapper.updatePsQuestionBatch(psqs);
            //批改完修改学生总得分和批改状态 (结构性试题)
            updateStuScore(psid,checkid);
        } catch (Exception e) {
            log.error("批改作业失败："+e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            throw new Exception("批改作业失败");
        }
    }

    /**
     * 作业批改完成之后一系列操作
     * @Param [psid]
     * @Author ywx
     * @Date 2020/6/11 11:08
     * @return void
     **/
    /*private void checkedOverDone(Integer psid) {
        //查询该作业该学生是否还存在未批改的题
        int psqStudentCount = paperStudentQuestionService.getNotCorrect(psid);
        if (psqStudentCount <= 0) {//该学生作业已经全部批改完成
            //作业学生关联表，学生批改状态置为已批改
            updateState(null, 1, psid, 3);
        }
    }*/

    /**
     * 批阅列表
     * @Param [ps]
     * @Author ywx
     * @Date 2020/6/18 14:34
     * @return java.util.List<com.xhkjedu.vo.paperstudent.PaperStudentVo>
     **/
    public List<PaperStudentVo> correctList(TPaperStudent ps) {
        return paperStudentMapper.correctList(ps);
    }

    //题库作业--按题批阅列表
    public List<Map> qcorrectList(Integer paperid, Integer classid) {
        List<Map> questions = paperStudentQuestionService.qcorrectList(paperid, classid);
        return questions;
    }

    //题库作业--按题批阅
    public void qcorrect(TPaperStudentQuestion psq) throws Exception {
        try {
            psq.setChecked(1);
            psq.setChecktime(N_Utils.getSecondTimestamp());
            paperStudentQuestionService.update(psq);

            //获取学生作业未批改试题数量
            Integer noCheckNum = paperStudentQuestionService.getNoCheckNumByPsId(psq.getPsid());
            if (noCheckNum.equals(0)) {
                //批改完修改学生总得分和批改状态 (结构性试题)
                updateStuScore(psq.getPsid(), psq.getCheckid());
            }
        } catch (Exception e) {
            log.error("批改作业失败："+e.getMessage());
            throw new Exception("批改失败");
        }
    }

    /**
     * 作业说明
     * @Param [psid]
     * @Author ywx
     * @Date 2020/6/18 17:10
     * @return com.xhkjedu.vo.paperstudent.PaperCommVo
     **/
    public PaperCommVo getPaperComm(Integer psid){
        return paperStudentMapper.getPaperComm(psid);
    }

    /**
     *功能描述 修改学生作答记录（教师添加批注）
     * @author WN
     * @date 2020/8/24
     * @param  * @param useranswer
     * @param psqid
     * @return void
     */
    public void updateUseranswer(String useranswer,Integer psqid){
        paperStudentQuestionService.updateUseranswer(useranswer,psqid);
    }

    //////////////////////////////////////////////图片作业 start//////////////////////////////////////////////////////////////////////

    /**
     *功能描述 图片作业-学生开始做作业
     * @author WN
     * @date 2020/6/12
     * @param  * @param psid
     * @return java.lang.Integer
     */
    public synchronized List<PaperStudentScantronVo> scantronStartPaper(Integer psid){
        List<PaperStudentScantronVo> psscanlst = listPaperStudentScantrons(psid);

        if(psscanlst.size()<=0){
            //保存学生关联答题卡信息
            paperStudentScantronService.saveBatchStudentScantron(psid);
            psscanlst = listPaperStudentScantrons(psid);
        }
        //更改开始状态
        updateState(null,null,psid,1);
        return psscanlst;
    }

    /**
     *功能描述 图片作业--学生作业答题卡
     * @author WN
     * @date 2020/6/12
     * @param  * @param psid
     * @return java.util.List<com.xhkjedu.vo.paperstudent.PaperStudentScantronVo>
     */
    public List<PaperStudentScantronVo> listPaperStudentScantrons(Integer psid) {
        return paperStudentScantronService.listPaperStudentScantrons(psid);
    }

    /**
     * 获取学生答题卡待批改试题列表
     * @Param [psid]
     * @Author ywx
     * @Date 2020/6/26 14:52
     * @return java.util.List<com.xhkjedu.vo.paperstudent.PaperStudentScantronVo>
     **/
    public List<PaperStudentScantronVo> sListCorrectQuestion(Integer psid) {
        return paperStudentScantronService.sListCorrectQuestion(psid);
    }

    /**
     *功能描述  图片作业--学生提交单题
     * @author WN
     * @date 2020/6/12
     * @param  * @param pss
     * @return int
     */
    public int scantronSubmitQuestion(TPaperStudentScantron pss) throws Exception {
        try {
            //学生作业试题
            TPaperStudentScantron pss1 = paperStudentScantronService.getObjById(pss.getPssid());
            if (pss1 == null) throw new Exception("该学生作业不存在");
            //获取原试题情况
            TPaperScantron scantronBean = paperScantronMapper.findById(pss1.getPscaid());
            pss.setChecked(0);
            pss.setAnswertime(N_Utils.getSecondTimestamp());
            //判断是否作答1.提交但未作答设置零分，2.作答设置答题状态为2，3.选择题提交为空判错
            String useranswer = pss.getUseranswer();
            List<String> userAnswers = JSON.parseArray(useranswer, String.class);
            String psanswer = scantronBean.getPsanswer();
            List<String> psanswers = JSON.parseArray(psanswer, String.class);
            //计算用户得分
            Double userScore = 0.0;
            //设置保留位数
            DecimalFormat df=new DecimalFormat("0.00");
            if (!N_Utils.isNotEmpty(pss.getUseranswer()) || userAnswers == null || userAnswers.size() <= 0) {
                pss.setStuscore(userScore);
                pss.setChecked(1);
                pss.setCheckid(0);
                pss.setChecktime(N_Utils.getSecondTimestamp());
                pss.setAnswered(0);
            } else {
                pss.setAnswered(1);
                //自动批改判断得分
                Integer ctype = scantronBean.getCtype();
                if (Arrays.asList(QTYPE).contains(ctype)) {
                    if (ctype == 1) {//单选
                        if (useranswer.equalsIgnoreCase(psanswer)) {
                            userScore = pss1.getQscore();
                        }
                    } else if (ctype == 2) {//多选
                        //多选不得分,或者选错不得分，不用处理
                        if (useranswer.length() <= psanswer.length()){
                            String[] qanswerList = psanswers.get(0).split("");
                            String[] userAnswerList = userAnswers.get(0).split("");
                            int rightNum = 0;//选对个数
                            for (String userAnswer : userAnswerList){
                                int right = 0;
                                for (String qanswer : qanswerList){
                                    if (qanswer.equals(userAnswer)){
                                        right = 1;
                                        rightNum += 1;
                                    }
                                }
                                if (right == 0){//选错直接0分，结束循环
                                    rightNum = 0;
                                    break;
                                }
                            }
                            if (rightNum > 0){//选对计算得分
                                if (qanswerList.length == rightNum) {//1.全选对满分
                                    userScore = pss1.getQscore();
                                } else if (qanswerList.length > rightNum) {//2.少选得一半分
                                    userScore = ArithUtil.div(pss1.getQscore(), 2 ,1);
                                }
                            }
                        }
                    } else if (ctype == 4 || ctype == 5 || ctype == 6) {
                        //判断题
                        if (useranswer.equalsIgnoreCase(psanswer)) {
                            userScore = pss1.getQscore();
                        }
                    } else if (ctype == 7 || ctype == 8 || ctype == 10) {//完型填空、阅读理解、任务型阅读
                        //计算每小题分数
                        Double qscore = ArithUtil.div(pss1.getQscore(), psanswers.size());
                        Double userScoreDouble = 0D;
                        for (int i = 0;i<psanswers.size();i++) {
                            if (psanswers.get(i).equalsIgnoreCase(userAnswers.get(i))) {
                                userScoreDouble = ArithUtil.add(userScoreDouble, qscore);
                            }
                        }
                        userScore = ArithUtil.round(userScoreDouble, 1);
                    }
                    if (userScore.compareTo(pss1.getQscore()) == 1){
                        userScore = pss1.getQscore();
                    }
                    pss.setStuscore(userScore);
                    pss.setChecked(1);
                    pss.setCheckid(0);
                    pss.setChecktime(N_Utils.getSecondTimestamp());
                }
            }

            Integer costtime = pss1.getCosttime();
            if (costtime == null) costtime = 0;
            pss.setCosttime(pss.getCosttime()+costtime);
            return paperStudentScantronService.updateStudentScantron(pss);
//            updateScantronStuScore(pss1.getPsid());//自动批改单题提交时修改学生总得分
        } catch (Exception e) {
            log.error("图片作业提交单题失败："+e.getMessage());
            throw new Exception("提交失败");
        }
    }

    /**
     *功能描述 图片作业--提交作业
     * @author WN
     * @date 2020/6/12
     * @param  * @param psid
     * @return int
     */
    public int scantronSubmitPaper(Integer psid) throws Exception {
        try {
            TPaperStudent ps = findById(psid);
            //如果学生作业不存在或是已提交，直接返回1提交成功
            if (ps == null || ps.getPsstate() != 0 ) return 1;
            Integer paperid = ps.getPaperid();
            Integer classid = ps.getClassid();
            TPaperStudent paperStudentBean = new TPaperStudent();
            paperStudentBean.setPsid(psid);
            int nowTime = N_Utils.getSecondTimestamp();
            TPaperGroup pg = paperGroupMapper.selectByPaperAndClassId(paperid, classid);
            if (pg.getStoptime() < nowTime) paperStudentBean.setPsstate(2);
            else paperStudentBean.setPsstate(1);
            paperStudentBean.setStoptime(nowTime);
            int consttime = paperStudentScantronService.getSumScantronsCosttime(psid);
            paperStudentBean.setCosttime(consttime);

            //答案显示方式：1作业截至时间，2提交后显示
            Integer showanswer = 0;
            if (pg.getShowtype() == 2){
                showanswer = 1;
            }
            paperStudentBean.setShowanswer(showanswer);
            return updateSPaper(psid, paperStudentBean, pg);
        } catch (Exception e) {
            log.error("图片作业提交作业失败："+e.getMessage());
            throw new Exception("提交失败");
        }
    }

    /**
     * 附件作业提交更新操作
     * @Param [psid, paperStudentBean, pg]
     * @Author ywx
     * @Date 2020/11/7 9:56
     * @return java.lang.Integer
     **/
    @Transactional(rollbackFor = Exception.class)
    public Integer updateSPaper(Integer psid, TPaperStudent paperStudentBean, TPaperGroup pg) throws Exception {
        Integer result = null;
        try {
            //把学生未做的题设置分值为0，状态为已批改
            paperStudentScantronService.updateScantronPSQByNotDo(psid);
            int psqStudentCount = paperStudentScantronService.getScantronNotCorrect(psid);
            Integer pschecked = 0;
            if (psqStudentCount <= 0) {//该学生作业已经全部批改完成
                //作业学生关联表，学生批改状态置为已批改
                pschecked = 1;
            }
            paperStudentBean.setPschecked(pschecked);
            result = paperStudentMapper.updateSPaper(paperStudentBean);
            questionClassService.submitPaper(pg);//自动批改的作业全部批改完调用批阅结束方法
        } catch (Exception e) {
            log.error("图片作业提交作业失败："+e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            throw new Exception("提交失败");
        }
        return result;
    }

    //学生作业处理
    /*private void sacntronCheckedOverDone(Integer psid) {
        //查询该作业该学生是否还存在未批改的题
        int psqStudentCount = paperStudentScantronService.getScantronNotCorrect(psid);
        if (psqStudentCount <= 0) {//该学生作业已经全部批改完成
            //作业学生关联表，学生批改状态置为已批改
            updateState(null, 1, psid, 3);
        }
    }*/

    /**
     * 批改完修改学生总得分和批改状态 (答题卡试题)
     * @author WN
     * @date 2020/6/12
     * @param psid
     * @param checkid
     * @return java.lang.Integer
     */
    public Integer updateScantronStuScore(Integer psid, Integer checkid){
        return paperStudentMapper.updateScantronStuScore(psid, checkid);
    }

    /**
     *功能描述 图片作业--单人批改
     * @author WN
     * @date 2020/6/12
     * @param  * @param psid
     * @param checkid
     * @param questionResult
     * @return int
     */
    public void scantronCorrect(Integer psid, Integer checkid, String questionResult) throws Exception {
        try {
            List<TPaperStudentScantron> psslist = JSON.parseArray(questionResult,TPaperStudentScantron.class);
            for (TPaperStudentScantron pss : psslist) {
                //设置批改状态为已批改
                pss.setCheckid(checkid);
                pss.setChecked(1);
                pss.setChecktime(N_Utils.getSecondTimestamp());
            }
            updateSCorrect(psid, checkid, psslist);
        } catch (Exception e) {
            log.error("图片作业批改失败："+e.getMessage());
            throw new Exception("批改失败");
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void updateSCorrect(Integer psid, Integer checkid, List<TPaperStudentScantron> psslist) throws Exception {
        try {
            //批量更新试题批改信息
            paperStudentScantronService.updateScantronQuestionBatch(psslist);
            //批改完修改学生总得分和批改状态 (答题卡试题)
            updateScantronStuScore(psid, checkid);
        } catch (Exception e) {
            log.error("图片作业批改失败："+e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            throw new Exception("图片作业批改失败");
        }
    }

    /**
     *功能描述 修改学生作答记录（教师添加批注）
     * @author WN
     * @date 2020/8/24
     * @param  * @param useranswer
     * @param psqid
     * @return void
     */
    public void updateScantronUseranswer(String useranswer,Integer psqid){
        paperStudentScantronService.updateUseranswer(useranswer,psqid);
    }

    /**
     *功能描述  修改作业答案是否显示
     * @author WN
     * @date 2020/10/13
     * @param  * @param
     * @return void
     */
    public void updateShowanswer(){
        paperStudentMapper.updateShowanswer();
    }



    //////////////////////////////////////////////图片作业 end//////////////////////////////////////////////////////////////////////

    /**
     * 根据学生作业id获取作业是否完成批阅
     * @Param [psid]
     * @Author ywx
     * @Date 2020/8/26 14:56
     * @return java.lang.Integer
     **/
    public Integer getCheckedByPsId(Integer psid){
        return paperGroupMapper.getCheckedByPsId(psid);
    }

    //学生作业退回
    public ResultVo backPaper(Integer psid) {
        TPaperStudent ps2 = new TPaperStudent();
        ps2.setPsid(psid);
        ps2.setPsstate(0);
        ps2.setPschecked(0);
        paperStudentMapper.updateByPrimaryKeySelective(ps2);
        return new ResultVo(0, "退回成功");
    }

    //获取学生作业状态
    public TPaperStudent getPaperState(Integer psid) {
        return paperStudentMapper.findById(psid);
    }

    public List<Map> sqcorrectList(Integer paperid, Integer classid) {
        return paperStudentScantronService.sqcorrectList(paperid, classid);
    }

    //图片作业--按题批改
    public void sqcorrect(TPaperStudentScantron pss) throws Exception {
        try {
            pss.setChecked(1);
            pss.setChecktime(N_Utils.getSecondTimestamp());
            paperStudentScantronService.updateStudentScantron(pss);

            //获取学生作业未批改试题数量
            Integer noCheckNum = paperStudentScantronService.getNoCheckNumByPsId(pss.getPsid());
            if (noCheckNum.equals(0)) {
                //批改完修改学生总得分和批改状态 (答题卡试题)
                updateScantronStuScore(pss.getPsid(), pss.getCheckid());
            }
        } catch (Exception e) {
            log.error("图片作业批改失败："+e.getMessage());
            throw new Exception("批改失败");
        }
    }
}
