package com.xhkjedu.sstudy.service.question;

import com.xhkjedu.sstudy.mapper.paper.PaperFileMapper;
import com.xhkjedu.sstudy.mapper.question.QuestionClassMapper;
import com.xhkjedu.sstudy.mapper.question.QuestionExplainMapper;
import com.xhkjedu.sstudy.mapper.question.QuestionStudentMapper;
import com.xhkjedu.sstudy.model.paper.TPaperFile;
import com.xhkjedu.sstudy.model.question.TQuestionExplain;
import com.xhkjedu.sstudy.model.question.TQuestionStudent;
import com.xhkjedu.sstudy.vo.question.QuestionClassVo;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.sstudy.vo.question.QuestionStudentVo;
import com.xhkjedu.sstudy.vo.subjectbook.SubjectQuestiontypeVo;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author ywx
 * @classname QuestionStudentService
 * @description 
 * @date 2020/6/6 17:34
 **/
@Service
public class QuestionStudentService {
    @Resource
    private QuestionStudentMapper questionStudentMapper;

    @Resource
    private QuestionExplainMapper questionExplainMapper;

    @Resource
    private PaperFileMapper paperFileMapper;
    @Resource
    private QuestionClassMapper questionClassMapper;

    //添加
    public Integer save(TQuestionStudent model) {
        questionStudentMapper.insertUseGeneratedKeys(model);
        return model.getId();
    }

    //删除
    public Integer deleteById(Integer qsid) {
        return questionStudentMapper.deleteById(qsid);
    }

    /**
     * 更新错误次数
     * @Param [qsid]
     * @Author ywx
     * @Date 2020/6/11 15:00
     * @return java.lang.Integer
     **/
    public Integer updateErrorNum(Integer qsid) {
        return questionStudentMapper.updateErrorNum(qsid);
    }

    //获取详情
    public TQuestionStudent findById(Integer id) {
        return questionStudentMapper.selectByPrimaryKey(id);
    }

    //获取列表
    public List <QuestionStudentVo> findAll(TQuestionStudent questionStudent) {
        List<QuestionStudentVo> list = questionStudentMapper.findAll(questionStudent);
        for (QuestionStudentVo q : list) {
            Integer qctype = q.getQctype();
            if (qctype == 1) {
                QuestionClassVo vo = questionClassMapper.getQuestionById(q.getQuestionid());
                q.setCtype(vo.getCtype());
                q.setQstem(vo.getQstem());
                q.setQoption(vo.getQoption());
                q.setQtypename(vo.getQtypename());
            } else if (qctype == 2) {
                QuestionClassVo vo = questionClassMapper.getScantronById(q.getQuestionid());
                q.setCtype(vo.getCtype());
                q.setQtypename(vo.getQtypename());
                String papername = questionStudentMapper.getPaperNameById(q.getPaperid());
                q.setQstem(papername+"-第"+q.getQorder()+"题");
            }
        }
        return list;
    }

    /**
     * 获取学生错题信息
     * @Param [questionid, studentid]
     * @Author ywx
     * @Date 2020/6/10 16:23
     * @return com.xhkjedu.model.question.TQuestionStudent
     **/
    public TQuestionStudent getQuestionStudent(TQuestionStudent qs) {
        return questionStudentMapper.getQuestionStudent(qs);
    }

    /**
     * 作答
     * @Param [qsid, corrected]
     * @Author ywx
     * @Date 2020/6/11 15:03
     * @return void
     **/
    public void submitQuestion(Integer qsid, Integer corrected) {
        if (corrected == 1){//作答正确，移除错题库
            deleteById(qsid);
        } else {//作答错误，更新错误次数
            updateErrorNum(qsid);
        }
    }

    //////////////////////////////////////////////学生WEB端//////////////////////////////////////////////
    /**
     * 未做提交或者自动批改未得满分，入错题库
     * @Param [questionid, studentid, subjectid]
     * @Author ywx
     * @Date 2020/6/10 16:26
     * @return java.lang.Boolean
     **/
    public Boolean updateQuestionStudent(TQuestionStudent qs){
        if (N_Utils.isEmpty(qs.getQuestionid())){//试题id为空不入错题库
            return true;
        }
        qs.setScorerate(N_Utils.getDoubleDivideAndMulitiply(qs.getStuscore(),qs.getQscore()));
        qs.setHandletime(N_Utils.getSecondTimestamp());
        TQuestionStudent questionStudent = getQuestionStudent(qs);
        if (questionStudent == null){
            qs.setErrornum(1);
            qs.setDelstate(0);
            try {
                Integer result = save(qs);
                return result > 0 ? true : false;
            } catch (DuplicateKeyException e) {
                return true;
            }
        }else {
            qs.setQsid(questionStudent.getQsid());
            Integer result = questionStudentMapper.updateQuestion(qs);
            return result > 0 ? true : false;
        }
    }

    /**
     * 已批阅未得满分的试题
     * @Param [paperid, classid, qctype]
     * @Author ywx
     * @Date 2020/10/12 15:10
     * @return java.util.List<com.xhkjedu.model.question.TQuestionStudent>
     **/
    public List<TQuestionStudent> listQuestions(Long paperid, Integer classid, Integer qctype) {
        return questionStudentMapper.listQuestions(paperid, classid, qctype);
    }

    /**
     * 题型列表
     * @Param [qs]
     * @Author ywx
     * @Date 2020/10/21 16:15
     * @return java.util.List<com.xhkjedu.vo.subjectbook.SubjectQuestiontypeVo>
     **/
    public List<SubjectQuestiontypeVo> listQtype(TQuestionStudent qs) {
        return questionStudentMapper.listQtype(qs);
    }

    //获取列表
    public List <QuestionStudentVo> findAllStuWeb(TQuestionStudent question) {
        List<QuestionStudentVo> list = questionStudentMapper.listForWeb(question);
        for (QuestionStudentVo q : list) {
            Integer qctype = q.getQctype();
            if (qctype == 1) {
                QuestionClassVo vo = questionClassMapper.getQuestionById(q.getQuestionid());
                q.setCtype(vo.getCtype());
                q.setQstem(vo.getQstem());
                q.setQoption(vo.getQoption());
                q.setQtypename(vo.getQtypename());
            } else if (qctype == 2) {
                QuestionClassVo vo = questionClassMapper.getScantronById(q.getQuestionid());
                q.setCtype(vo.getCtype());
                q.setQtypename(vo.getQtypename());
            }
            List<TQuestionExplain> explains = questionExplainMapper.listByQuestionId(q.getQuestionid(), q.getQctype());
            q.setExplains(explains);
        }
        return list;
    }

    /**
     * 试题详情
     * @Param [questionid, qctype, studentid]
     * @Author ywx
     * @Date 2020/9/5 10:12
     * @return com.xhkjedu.vo.question.QuestionStudentVo
     **/
    public QuestionStudentVo detailStuWeb(String questionid, Integer qctype, Integer studentid) {
        QuestionStudentVo question = null;
        if (qctype == 1){
            question = questionStudentMapper.detail(questionid, studentid);
        } else {
            question = questionStudentMapper.sdetail(questionid, studentid);
            if (question != null) {
                List<TPaperFile> files = paperFileMapper.listObjsByPaperid(question.getPaperid());
                question.setFiles(files);
            }
        }
        return question;
    }

}
