package com.xhkjedu.suser.service.gradeclass;

import com.xhkjedu.config.MissingParametersException;
import com.xhkjedu.suser.mapper.gradeclass.ClassMapper;
import com.xhkjedu.suser.mapper.gradeclass.ClassStudentMapper;
import com.xhkjedu.suser.mapper.gradeclass.SchoolMapper;
import com.xhkjedu.suser.mapper.liveplay.CourseMapper;
import com.xhkjedu.suser.model.gradeclass.TClassStudent;
import com.xhkjedu.suser.model.gradeclass.TGroupStudent;
import com.xhkjedu.suser.model.system.TUser;
import com.xhkjedu.suser.service.system.UserService;
import com.xhkjedu.suser.utils.PoiUtils;
import com.xhkjedu.suser.vo.gradeclass.ClassStudentVo;
import com.xhkjedu.suser.vo.gradeclass.ClassVo;
import com.xhkjedu.suser.vo.gradeclass.ImportStudentVo;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.vo.ResultVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author ywx
 * @classname TClassStudentService
 * @description
 * @date 2020/6/2 15:53
 **/
@Service
@Slf4j
public class ClassStudentService {
    @Resource
    private ClassStudentMapper classStudentMapper;
    @Resource
    private UserService userService;
    @Resource
    private GroupStudentService groupStudentService;
    @Resource
    private ClassMapper classMapper;
    @Resource
    private SchoolMapper schoolMapper;
    @Resource
    private CourseMapper courseMapper;

    //批量添加
    public void batchAdd(List<TClassStudent> students) throws Exception {
        for (TClassStudent student : students) {
            save(student);
        }
    }

    //班级学生批量添加-取消的删除
    public void batchAdd2(Integer classid, List<Integer> newStuIds) throws Exception {
        List<Integer> studentIds = classStudentMapper.listStudentIdByClassId(classid);//班级原有学生
        for (Integer studentid : newStuIds) {
            if (!N_Utils.numInList(studentid, studentIds)) {//新添加的学生
                save2(classid, studentid);
            }
        }
        for (Integer studentId : studentIds) {
            if (!N_Utils.numInList(studentId, newStuIds)) {//删除的学生
                deleteById(classid, studentId);
            }
        }
    }

    //添加
    @Transactional(rollbackFor = Exception.class)
    public Integer save(TClassStudent model) throws Exception {
        model.setCreatetime(N_Utils.getSecondTimestamp());
        if (!N_Utils.isEmpty(model.getStudentno())) {
            String headpic = model.getHeadpic();
            TUser user = userService.getObjByLoginName(model.getStudentno());
            if (user == null) {//学生信息在用户表不存在，添加学生信息
                user = new TUser();
                user.setUsertype(2);
                user.setUsername(model.getStudentname());
                user.setLoginname(model.getStudentno());
                user.setLoginpwd("123456");
                user.setUsersex(model.getUsersex());
                user.setStudentno(model.getStudentno());
                user.setCreateid(model.getCreateid());
                user.setSchoolid(model.getSchoolid());
                user.setUsertime(model.getUsertime());
                user.setHeadpic(headpic);
                ResultVo vo = userService.save(user);
                if (vo.getCode() != 0) throw new MissingParametersException(vo.getMsg());
                user.setUserid((Integer) vo.getObj());
            } else if (user.getUsertype() != 2) {
                throw new Exception("禁止添加非学生账号");
            } else if (!model.getSchoolid().equals(user.getSchoolid())) {
                throw new Exception("禁止添加非本校学生账号");
            } else if (N_Utils.isNotEmpty(headpic)) {
                user.setHeadpic(headpic);
                userService.updateHeadPic(user);
            }

            model.setStudentid(user.getUserid());
        }

        //全体分组
        ClassVo classVo = classMapper.getById(model.getClassid());
        ImportStudentVo student = getNumById(model.getClassid(), model.getStudentid(), classVo.getYear());
        if (classVo.getClasstype() == 1 && student.getXznum() != 0) {
            throw new MissingParametersException("该学生已在行政班");
        }
        Integer num = student.getClassnum();//班级学生数量
        if (num == 0) {//学生不在该班级
            model.setCreateid(classVo.getCreateid());
            model.setSchoolid(classVo.getSchoolid());
            classStudentMapper.insertUseGeneratedKeys(model);//添加班级学生
            classMapper.updateNum(model.getClassid());//更新学生数量

            //分组学生
            Integer groupid = classVo.getGroupid();
            TGroupStudent g = new TGroupStudent();
            g.setClassid(model.getClassid());
            g.setGroupid(groupid);
            g.setStudentid(model.getStudentid());
            g.setCreateid(model.getCreateid());
            g.setSchoolid(model.getSchoolid());
            groupStudentService.save(g);

            //一般分组
            if (!N_Utils.isEmptyInteger(model.getGroupid())) {
                TGroupStudent groupStudent = new TGroupStudent();
                groupStudent.setClassid(model.getClassid());
                groupStudent.setGroupid(model.getGroupid());
                groupStudent.setStudentid(model.getStudentid());
                groupStudent.setCreateid(model.getCreateid());
                groupStudent.setSchoolid(model.getSchoolid());
                groupStudentService.save(groupStudent);
            }
        }

        //把这个学生加到此行政班、提高班相关的所有直播班中
        if (classVo.getClasstype() == 1 || classVo.getClasstype() == 2) {
            saveCourseStu(model.getClassid(), model.getStudentid());
        }
        return model.getId();
    }

    //把这个学生加到此行政班相关的所有直播班中
    public void saveCourseStu(Integer classid, Integer studentid) {
        List<Map> courses = courseMapper.listXzCourse(classid);
        for (Map course : courses) {
            Integer courseid = (Integer) course.get("courseid");
            saveSelection((Integer) course.get("classid"), studentid, (Integer) course.get("createid"), (Integer) course.get("schoolid"));
            courseMapper.updateStuNum(courseid);
        }
    }

    //学生选课
    public void saveSelection(Integer classid, Integer studentid, Integer teacherid, Integer schoolid) {
        TClassStudent student = classStudentMapper.getByClassIdStudentId(classid, studentid);
        if (student == null) {
            //班级学生
            TClassStudent cs = new TClassStudent();
            cs.setClassid(classid);
            cs.setStudentid(studentid);
            cs.setCreateid(teacherid);
            cs.setSchoolid(schoolid);
            cs.setCreatetime(N_Utils.getSecondTimestamp());
            classStudentMapper.save(cs);

            //分组学生
            ClassVo classVo = classMapper.getById(classid);
            Integer groupid = classVo.getGroupid();
            TGroupStudent gs = new TGroupStudent();
            gs.setClassid(classid);
            gs.setGroupid(groupid);
            gs.setStudentid(studentid);
            gs.setCreateid(teacherid);
            gs.setSchoolid(schoolid);
            gs.setCreatetime(N_Utils.getSecondTimestamp());
            groupStudentService.save(gs);
        }

        //更新班级学生数量
        classMapper.updateNum(classid);
    }

    //学生取消选课
    public void cancelSelection(Integer classid, Integer studentid) {
        classStudentMapper.deleteById(classid, studentid);
        //更新班级、分组学生数量
        classMapper.updateNum2(classid);
    }

    //删除
    @Transactional(rollbackFor = Exception.class)
    public void deleteById(Integer classid, Integer studentid) {
        classStudentMapper.deleteById(classid, studentid);//删除班级、分组对应学生
        classMapper.updateNum2(classid);//更新班级、分组学生数量
        //把这个学生从此行政班、提高班关联的所有直播班移除
        ClassVo classVo = classMapper.getById(classid);
        if (classVo.getClasstype() == 1 || classVo.getClasstype() == 2) {
            List<Map> courses = courseMapper.listXzCourse(classid);
            for (Map course : courses) {
                cancelSelection((Integer) course.get("classid"), studentid);
            }
        }
    }

    //修改
    public ResultVo update(TClassStudent model) {
        TUser u = userService.getObjByLoginName(model.getStudentno());
        if (u != null && !model.getStudentid().equals(u.getUserid())) {
            return new ResultVo(1, "该账号已被【" + u.getUsername() + "】占用");
        }
        //修改学生信息
        TUser user = new TUser();
        user.setUserid(model.getStudentid());
        user.setUsersex(model.getUsersex());
        user.setUsername(model.getStudentname());
        user.setStudentno(model.getStudentno());
        user.setLoginname(model.getStudentno());
        user.setUserphone(model.getUserphone());
        user.setUsertime(model.getUsertime());
        user.setHeadpic(model.getHeadpic());
        userService.update(user);
        return new ResultVo(0, "修改成功");
    }

    //获取列表
    public List<ClassStudentVo> findAll(TClassStudent student) {
        setOrderField(student);
        return classStudentMapper.findAll(student);
    }

    //班级学生导入
    @Transactional(rollbackFor = Exception.class)
    public ResultVo importUser(MultipartFile file, TClassStudent tClassStudent) {
        Integer schoolid = tClassStudent.getSchoolid();
        Integer createid = tClassStudent.getCreateid();
        N_Utils.validation(new Object[]{schoolid, "学校id", 1, createid, "创建人id", 1});

        try {

            //学校所有正常行政班
            Map<Integer, List<Map<String, Integer[]>>> gcg = listGradeClassGroup(schoolid, 1);

            Map<String, Object> map = PoiUtils.readExcelStudent(file, gcg);
            List<TUser> students = (List<TUser>) map.get("users");
            map.remove("users");//导入成功的学生信息不返回给前端
            /*if (!CollectionUtils.isEmpty((List<TUser>)map.get("excelExisted"))) {
                map.put("code",1);
                return new ResultVo(0, "导入失败", map);
            }*/

            List<TUser> users = new ArrayList<>();
            List<TClassStudent> classStudents = new ArrayList<>();
            List<TGroupStudent> groupStudents = new ArrayList<>();
            // 以下学生信息已存在
            List<TUser> usersExisted = new ArrayList<>();
            for (TUser user : students) {
                TUser u = userService.getObjByLoginName(user.getLoginname());
                if (null == u) {
                    user.setSchoolid(schoolid);
                    user.setCreateid(createid);
                    user.setCreatetime(N_Utils.getSecondTimestamp());
                    user.setUserstate(1);
                    user.setUsertype(2);
                    users.add(user);
                } else {
                    Integer studentid = u.getUserid();
                    Integer oldclassid = classStudentMapper.getClassIdByStuId(studentid);//获取学生行政班id
                    if (2 != u.getUsertype() || schoolid != u.getSchoolid()) {//非本校学生信息
                        usersExisted.add(u);
                    } else if (!user.getClassid().equals(oldclassid) && 2 == u.getUsertype() && schoolid == u.getSchoolid()) {
                        if (1 != u.getUserstate()) {
                            userService.updateState(u.getUserid(), 1);
                        }
                        if (oldclassid == null) {//学生不在行政班
                            setStudents(user.getClassid(), schoolid, createid, user.getGroupid(), classStudents, groupStudents, studentid);
                        } else {
                            moveClass(oldclassid, user.getClassid(), studentid);//学生在其他行政班转班
                        }
                    } else {//本班学生信息
                        usersExisted.add(u);
                    }
                }
            }

            //保存用户信息
            if (!CollectionUtils.isEmpty(users)) {
                userService.batchSave(users);
                for (TUser user : users) {
                    Integer studentid = user.getId();
                    setStudents(user.getClassid(), schoolid, createid, user.getGroupid(), classStudents, groupStudents, studentid);
                }
            }

            //保存学生关联信息
            if (!CollectionUtils.isEmpty(classStudents)) {
                batchSave(classStudents);
                groupStudentService.batchSave(groupStudents);

                //获取所有导入学生的班级
                List classidlist = students.stream().map(TUser::getClassid).collect(Collectors.toList());
                String classids = (String) classidlist.stream().map(Object::toString).collect(Collectors.joining(","));
                classMapper.updateClassGroupNumByType(classids);//更新指定班级班级、分组学生数量
                for (TClassStudent classStudent : classStudents) {
                    saveCourseStu(classStudent.getClassid(), classStudent.getStudentid());
                }
            }
            map.put("usersExisted", usersExisted);
            if (N_Utils.isListNotEmpty(usersExisted)) {
                map.put("code", 1);
            } else {
                map.put("code", 0);
            }
            return new ResultVo(0, "导入成功", map);
        } catch (Exception e) {
            log.error("导入失败 : " + e.getMessage());
            return new ResultVo(1, "导入失败 : " + e.getMessage());
        }
    }

    //获取全校年级、班级、分组
    private Map<Integer, List<Map<String, Integer[]>>> listGradeClassGroup(Integer schoolid, Integer classtype) {
        Map<Integer, List<Map<String, Integer[]>>> gclasses = new HashMap<>();//年级下班级集合

        List<Map> classes = classMapper.listClass(schoolid, classtype);
        Map<Integer, List<Map>> grades = classes.stream().collect(Collectors.groupingBy(c -> (Integer) c.get("gradeid")));
        for (Map.Entry<Integer, List<Map>> g : grades.entrySet()) {
            List<Map<String, Integer[]>> cmap = new ArrayList<>();
            List<Map> list = g.getValue();
            for (Map m : list) {
                Map<String, Integer[]> c = new HashMap();
                Integer[] classid_groupid = {(Integer) m.get("classid"), (Integer) m.get("groupid"), (Integer) m.get("year")};
                c.put(m.get("classname").toString(), classid_groupid);

                cmap.add(c);
            }
            gclasses.put(g.getKey(), cmap);
        }
        return gclasses;
    }

    //提高班学生导入
    public ResultVo importUser2(MultipartFile file, TClassStudent tClassStudent) {
        Integer schoolid = tClassStudent.getSchoolid();
        Integer createid = tClassStudent.getCreateid();
        N_Utils.validation(new Object[]{schoolid, "学校id", 1, createid, "创建人id", 1});
        try {
            //学校所有正常提高班
            Map<Integer, List<Map<String, Integer[]>>> gcg = listGradeClassGroup(schoolid, 2);
            Map<String, Object> map = PoiUtils.readExcelStudent(file, gcg);
            List<TUser> students = (List<TUser>) map.get("users");
            map.remove("users");//导入成功的学生信息不返回给前端

            List<TClassStudent> classStudents = new ArrayList<>();
            List<TGroupStudent> groupStudents = new ArrayList<>();
            // 以下学生信息不存在
            List<TUser> usersNotExisted = new ArrayList<>();
            for (TUser user : students) {
                TUser u = userService.getObjByLoginName(user.getLoginname());
                if (null == u) {
                    user.setSchoolid(schoolid);
                    user.setCreateid(createid);
                    user.setCreatetime(N_Utils.getSecondTimestamp());
                    user.setUserstate(1);
                    user.setUsertype(2);
                    usersNotExisted.add(user);
                } else {
                    Integer studentid = u.getUserid();
                    ImportStudentVo student = getNumById(user.getClassid(), studentid, user.getYear());
                    Integer classnum = student.getClassnum();//本班级学生数量
                    if (2 != u.getUsertype() || schoolid != u.getSchoolid() || 0 == student.getXznum()) {
                        usersNotExisted.add(u);
                    } else if (0 == classnum && 2 == u.getUsertype() && schoolid == u.getSchoolid()) {
                        if (1 != u.getUserstate()) {
                            userService.updateState(u.getUserid(), 1);
                        }
                        setStudents(user.getClassid(), schoolid, createid, user.getGroupid(), classStudents, groupStudents, studentid);
                    }
                }
            }

            //保存学生关联信息
            if (!CollectionUtils.isEmpty(classStudents)) {
                batchSave(classStudents);
                groupStudentService.batchSave(groupStudents);
                //获取所有导入学生的班级
                List classidlist = students.stream().map(TUser::getClassid).collect(Collectors.toList());
                String classids = (String) classidlist.stream().map(Object::toString).collect(Collectors.joining(","));
                classMapper.updateClassGroupNumByType(classids);//更新指定班级班级、分组学生数量
            }
            map.put("usersNotExisted", usersNotExisted);
            if (N_Utils.isListNotEmpty(usersNotExisted)) {
                map.put("code", 1);
            } else {
                map.put("code", 0);
            }
            return new ResultVo(0, "导入成功", map);
        } catch (Exception e) {
            log.error("导入失败 : " + e.getMessage());
            return new ResultVo(1, "导入失败 : " + e.getMessage());
        }
    }

    private void setStudents(Integer classid, Integer schoolid, Integer createid, Integer groupid, List<TClassStudent> classStudents, List<TGroupStudent> groupStudents, Integer studentid) {
        TClassStudent cs = new TClassStudent();
        cs.setClassid(classid);
        cs.setStudentid(studentid);
        cs.setCreateid(createid);
        cs.setCreatetime(N_Utils.getSecondTimestamp());
        cs.setSchoolid(schoolid);
        classStudents.add(cs);

        TGroupStudent gs = new TGroupStudent();
        gs.setGroupid(groupid);
        gs.setClassid(classid);
        gs.setStudentid(studentid);
        gs.setCreateid(createid);
        gs.setCreatetime(N_Utils.getSecondTimestamp());
        gs.setSchoolid(schoolid);
        groupStudents.add(gs);
    }

    private void batchSave(List<TClassStudent> classStudents) {
        classStudentMapper.insertList(classStudents);
        //2021年4月28日，张道剑建议，南华凯、张剑、王宁 共同确认不再复制预习导学信息
        /*for (TClassStudent s : classStudents) {
            guideGroupStudentMapper.copyGuideResource(s.getStudentid(), s.getClassid());//复制学生预习导学信息
        }*/
    }

    //班级学生数量
    private ImportStudentVo getNumById(Integer classid, Integer studentid, Integer year) {
        return classStudentMapper.getNumById(classid, studentid, year);
    }

    /**
     * 转班
     *
     * @return void
     * @Param [oldclassid, classid, groupid, studentid]
     * @Author ywx
     * @Date 2020/6/19 10:11
     **/
    @Transactional(rollbackFor = Exception.class)
    public void moveClass(Integer oldclassid, Integer classid, Integer studentid) throws Exception {
        deleteById(oldclassid, studentid);//删除原班级、分组对应学生;更新原班级、分组学生数量
        save2(classid, studentid);//添加班级学生
        saveCourseStu(classid, studentid);
    }

    //添加班级学生
    private void save2(Integer classid, Integer studentid) {
        //全体分组
        ClassVo classVo = classMapper.getById(classid);
        //班级学生
        TClassStudent cs = new TClassStudent();
        cs.setClassid(classid);
        cs.setStudentid(studentid);
        Integer createid = classVo.getCreateid();
        cs.setCreateid(createid);
        cs.setCreatetime(N_Utils.getSecondTimestamp());
        Integer schoolid = classVo.getSchoolid();
        cs.setSchoolid(schoolid);
        classStudentMapper.insertUseGeneratedKeys(cs);//添加班级学生
        classMapper.updateNum(classid);//更新学生数量

        //分组学生
        Integer groupid = classVo.getGroupid();
        TGroupStudent g = new TGroupStudent();
        g.setClassid(classid);
        g.setGroupid(groupid);
        g.setStudentid(studentid);
        g.setCreateid(createid);
        g.setSchoolid(schoolid);
        groupStudentService.save(g);
    }

    /**
     * 解除mac绑定
     *
     * @return void
     * @Param [studentid]
     * @Author ywx
     * @Date 2020/9/18 11:56
     **/
    public void removeBind(Integer studentid) {
        userService.removeBind(studentid);
    }

    /**
     * 功能描述 获取学生所在班级
     *
     * @param *         @param year
     * @param studentid
     * @return java.util.List<com.xhkjedu.suser.vo.gradeclass.ClassVo>
     * @author WN
     * @date 2020/10/20
     */
    public List<ClassVo> listClassByStudentId(Integer studentid) {
        Integer year = schoolMapper.getYearByUserId(studentid);
        return classStudentMapper.listClassByStudentId(year, studentid);
    }

    public List<ClassStudentVo> listNoClassStu(TClassStudent student) {
        setOrderField(student);
        setYear(student);
        return classStudentMapper.listNoClassStu(student);
    }

    private void setOrderField(TClassStudent student) {
        String fieldname = student.getFieldname();
        if (N_Utils.isEmpty(fieldname) || fieldname.equals("createtime")) {
            fieldname = "cs.createtime";//按创建时间排序导入的学生分页显示会有重复
            if (N_Utils.isEmpty(student.getFieldorder())) {
                student.setFieldorder("desc");
            }
        } else if (fieldname.equals("studentname")) {
            fieldname = "convert(u.username using gbk)";
        } else if (fieldname.equals("studentno")) {
            fieldname = "u.studentno";
        }
        student.setFieldname(fieldname);
    }

    //获取学生所在年级列表
    public List<Integer> listGrade(Integer studentid) {
        Integer year = schoolMapper.getYearByUserId(studentid);
        return classStudentMapper.listGrade(studentid, year);
    }

    //获取全校学生列表
    public List<ClassStudentVo> listAllStudents(TClassStudent student) {
        return classStudentMapper.listAllStudents(student);
    }

    //设置年份
    public void setYear(TClassStudent student) {
        Integer year = schoolMapper.getYearById(student.getSchoolid());
        student.setYear(year);
    }

    //删除学生所有提高班和所在提高班直播课
    public ResultVo delTgStu(String loginname) {
        String[] userIds = loginname.split(",");
        for (String userid : userIds) {
            Integer studentid = N_Utils.str2Int(userid);
            List<Integer> classids = classStudentMapper.listTgClassid(studentid);//获取学生所有提高班
            for (Integer classid : classids) {
                deleteById(classid, studentid);
            }
        }
        return new ResultVo(0, "取消选课成功");
    }
}
